@@ -2,11 +2,13 @@ import numpy as np
 import xarray as xr
 from abc import abstractmethod
 from dask.distributed import progress
+from dask.diagnostics import ProgressBar
 
 from .model import Model
-from .data import Data
-import foxes.variables as FV
+from .data import MData, FData, TData
+from foxes.utils.runners import DaskRunner
 import foxes.constants as FC
+import foxes.variables as FV
 
 
 class DataCalcModel(Model):
@@ -22,6 +24,8 @@ class DataCalcModel(Model):
     For each individual data chunk the `calculate`
     function is called.
 
+    :group: core
+
     """
 
     @abstractmethod
@@ -34,16 +38,16 @@ class DataCalcModel(Model):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        *data : foxes.core.Data
+        data: tuple of foxes.core.Data
             The input data
-        **parameters : dict, optional
+        parameters: dict, optional
             The calculation parameters
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray
 
@@ -64,22 +68,35 @@ class DataCalcModel(Model):
         out_vars,
         out_dims,
         calc_pars,
+        init_vars,
     ):
         """
         Wrapper that mitigates between apply_ufunc and `calculate`.
         """
+        n_prev = len(init_vars)
+        if n_prev:
+            prev = ldata[:n_prev]
+            ldata = ldata[n_prev:]
 
         # reconstruct original data:
         data = []
-        for hvars in dvars:
-
+        for i, hvars in enumerate(dvars):
             v2l = {v: lvars.index(v) for v in hvars if v in lvars}
             v2e = {v: evars.index(v) for v in hvars if v in evars}
 
             hdata = {v: ldata[v2l[v]] if v in v2l else edata[v2e[v]] for v in hvars}
             hdims = {v: ldims[v2l[v]] if v in v2l else edims[v2e[v]] for v in hvars}
 
-            data.append(Data(hdata, hdims, loop_dims))
+            if i == 0:
+                data.append(MData(data=hdata, dims=hdims, loop_dims=loop_dims))
+            elif i == 1:
+                data.append(FData(data=hdata, dims=hdims, loop_dims=loop_dims))
+            elif i == 2:
+                data.append(TData(data=hdata, dims=hdims, loop_dims=loop_dims))
+            else:
+                raise NotImplementedError(
+                    f"Not more than 3 data sets implemented, found {len(dvars)}"
+                )
 
             del hdata, hdims, v2l, v2e
 
@@ -94,21 +111,40 @@ class DataCalcModel(Model):
                 raise ValueError(f"Model '{self.name}': Failed to find loop dimension")
 
         # add zero output data arrays:
-        odims = {v: out_dims for v in out_vars}
+        odims = {v: tuple(out_dims) for v in out_vars}
         odata = {
-            v: np.full(oshape, np.nan, dtype=FC.DTYPE)
+            v: (
+                np.full(oshape, np.nan, dtype=FC.DTYPE)
+                if v not in init_vars
+                else prev[init_vars.index(v)].copy()
+            )
             for v in out_vars
             if v not in data[-1]
         }
+
         if len(data) == 1:
-            data.append(Data(odata, odims, loop_dims))
+            data.append(FData(odata, odims, loop_dims))
         else:
             odata.update(data[-1])
             odims.update(data[-1].dims)
-            data[-1] = Data(odata, odims, loop_dims)
+            if len(data) == 2:
+                data[-1] = FData(odata, odims, loop_dims)
+            else:
+                data[-1] = TData(odata, odims, loop_dims)
         del odims, odata
 
+        # link chunk state indices from mdata to fdata and tdata:
+        if FC.STATE in data[0]:
+            for d in data[1:]:
+                d[FC.STATE] = data[0][FC.STATE]
+
+        # link weights from mdata to fdata:
+        if FV.WEIGHT in data[0]:
+            data[1][FV.WEIGHT] = data[0][FV.WEIGHT]
+            data[1].dims[FV.WEIGHT] = data[0].dims[FV.WEIGHT]
+
         # run model calculation:
+        self.ensure_variables(algo, *data)
         results = self.calculate(algo, *data, **calc_pars)
 
         # replace missing results by first input data with matching shape:
@@ -137,7 +173,16 @@ class DataCalcModel(Model):
         return data
 
     def run_calculation(
-        self, algo, *data, out_vars, loop_dims, out_core_vars, **calc_pars
+        self,
+        algo,
+        *data,
+        out_vars,
+        loop_dims,
+        out_core_vars,
+        initial_results=None,
+        sel=None,
+        isel=None,
+        **calc_pars,
     ):
         """
         Starts the model calculation in parallel, via
@@ -147,30 +192,38 @@ class DataCalcModel(Model):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        *data : tuple of xarray.Dataset
+        data: tuple of xarray.Dataset
             The input data
         out_vars: list of str
             The calculation output variables
-        loop_dims : array_like of str
+        loop_dims: array_like of str
             List of the loop dimensions during xarray's
             `apply_ufunc` calculations
-        out_core_vars : list of str
+        out_core_vars: list of str
             The core dimensions of the output data, use
-            `FV.VARS` for variables dimension (required)
-        **calc_pars : dict, optional
+            `FC.VARS` for variables dimension (required)
+        initial_results: xarray.Dataset, optional
+            Initial results
+        sel: dict, optional
+            Selection of loop_dim variable subset values
+        isel: dict, optional
+            Selection of loop_dim variable subset index values
+        calc_pars: dict, optional
             Additional arguments for the `calculate` function
 
         Returns
         -------
-        results : xarray.Dataset
+        results: xarray.Dataset
             The calculation results
 
         """
         # check:
         if not self.initialized:
-            raise ValueError(f"DataCalcModel '{self.name}': run_calculation called for uninitialized model")
+            raise ValueError(
+                f"DataCalcModel '{self.name}': run_calculation called for uninitialized model"
+            )
 
         # prepare:
         loopd = set(loop_dims)
@@ -183,8 +236,16 @@ class DataCalcModel(Model):
         evars = []
         edims = []
         dvars = []
-        for ds in data:
+        ivars = []
+        idims = []
+        if initial_results is not None:
+            ds = initial_results
+            hvarsl = [v for v, d in ds.items() if len(loopd.intersection(d.dims))]
+            ldata += [ds[v] for v in hvarsl]
+            idims += [ds[v].dims for v in hvarsl]
+            ivars += hvarsl
 
+        for ds in data:
             hvarsl = [v for v, d in ds.items() if len(loopd.intersection(d.dims))]
             ldata += [ds[v] for v in hvarsl]
             ldims += [ds[v].dims for v in hvarsl]
@@ -207,17 +268,35 @@ class DataCalcModel(Model):
 
             dvars.append(list(ds.keys()) + list(ds.coords.keys()))
 
+        # subset selection:
+        if sel is not None:
+            nldata = []
+            for ds in ldata:
+                s = {k: v for k, v in sel.items() if k in ds.coords}
+                if len(s):
+                    nldata.append(ds.sel(s))
+            ldata = nldata
+            del nldata
+        if isel is not None:
+            nldata = []
+            for ds in ldata:
+                s = {k: v for k, v in isel.items() if k in ds.coords}
+                if len(s):
+                    nldata.append(ds.isel(s))
+            ldata = nldata
+            del nldata
+
         # setup dask options:
-        dargs = dict(output_sizes={FV.VARS: len(out_vars)})
-        if FV.TURBINE in loopd and FV.TURBINE not in ldims.values():
-            dargs["output_sizes"][FV.TURBINE] = algo.n_turbines
-        if FV.VARS not in out_core_vars:
+        dargs = dict(output_sizes={FC.VARS: len(out_vars)})
+        if FC.TURBINE in loopd and FC.TURBINE not in ldims.values():
+            dargs["output_sizes"][FC.TURBINE] = algo.n_turbines
+        if FC.VARS not in out_core_vars:
             raise ValueError(
-                f"Model '{self.name}': Expecting '{FV.VARS}' in out_core_vars, got {out_core_vars}"
+                f"Model '{self.name}': Expecting '{FC.VARS}' in out_core_vars, got {out_core_vars}"
             )
 
         # setup arguments for wrapper function:
-        out_dims = loop_dims + list(set(out_core_vars).difference([FV.VARS]))
+        out_dims = loop_dims + list(set(out_core_vars).difference([FC.VARS]))
         wargs = dict(
             algo=algo,
             dvars=dvars,
@@ -230,14 +309,16 @@ class DataCalcModel(Model):
             out_vars=out_vars,
             out_dims=out_dims,
             calc_pars=calc_pars,
+            init_vars=ivars,
         )
 
         # run parallel computation:
+        iidims = [[c for c in d if c not in loopd] for d in idims]
         icdims = [[c for c in d if c not in loopd] for d in ldims]
         results = xr.apply_ufunc(
             self._wrap_calc,
             *ldata,
-            input_core_dims=icdims,
+            input_core_dims=iidims + icdims,
             output_core_dims=[out_core_vars],
             output_dtypes=[FC.DTYPE],
             dask="parallelized",
@@ -246,16 +327,10 @@ class DataCalcModel(Model):
         )
 
         # reorganize results Dataset:
-        results = (
-            results.assign_coords({FV.VARS: out_vars}).to_dataset(dim=FV.VARS).persist()
-        )
+        results = results.assign_coords({FC.VARS: out_vars}).to_dataset(dim=FC.VARS)
 
-        # try to show progress bar:
-        if algo.verbosity > 0:
-            try:
-                progress(results)
-            except ValueError:
-                pass
+        if DaskRunner.is_distributed() and len(ProgressBar.active):
+            progress(results.persist())
 
         # update data by calculation results:
         return results.compute()
