@@ -4,62 +4,78 @@ from .farm_data_model import FarmDataModelList, FarmDataModel
 from .turbine_model import TurbineModel
 from .turbine_type import TurbineType
 import foxes.constants as FC
-import foxes.variables as FV
 
 
 class FarmController(FarmDataModel):
     """
     Analyses selected turbine models and handles their call.
 
-    Parameters
-    ----------
-    pars : dict
-        Parameters for the turbine models, stored
-        under their respective name
-
     Attributes
     ----------
-    turbine_types : list of foxes.core.TurbineType
+    turbine_types: list of foxes.core.TurbineType
         The turbine type of each turbine
-    turbine_model_names : list of str
+    turbine_model_names: list of str
         Names of all turbine models found in the farm
-    turbine_model_sels : numpy.ndarray of bool
-        Selection flags for all turbine models,
-        shape: (n_states, n_turbines, n_models)
-    pre_rotor_models : foxes.core.FarmDataModelList
+    pre_rotor_models: foxes.core.FarmDataModelList
         The turbine models with pre-rotor flag
-    post_rotor_models : foxes.core.FarmDataModelList
+    post_rotor_models: foxes.core.FarmDataModelList
         The turbine models without pre-rotor flag
-    pars : dict
+    pars: dict
         Parameters for the turbine models, stored
         under their respecitve name
 
+    :group: core
+
     """
 
     def __init__(self, pars={}):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        pars: dict
+            Parameters for the turbine models, stored
+            under their respective name
+
+        """
         super().__init__()
 
         self.turbine_types = None
         self.turbine_model_names = None
-        self.turbine_model_sels = None
         self.pre_rotor_models = None
         self.post_rotor_models = None
 
         self.pars = pars
 
+    def sub_models(self):
+        """
+        List of all sub-models
+
+        Returns
+        -------
+        smdls: list of foxes.core.Model
+            Names of all sub models
+
+        """
+        return [
+            self.pre_rotor_models,
+            self.post_rotor_models,
+        ]
+
     def set_pars(self, model_name, init_pars, calc_pars, final_pars):
         """
         Set parameters for a turbine model
 
         Parameters
         ----------
-        model_name : str
+        model_name: str
             Name of the model
-        init_pars : dict
+        init_pars: dict
             Parameters for initialization
-        calc_pars : dict
+        calc_pars: dict
             Parameters for calculation
-        final_pars : dict
+        final_pars: dict
             Parameters for finalization
 
         """
@@ -82,9 +98,7 @@ class FarmController(FarmDataModel):
             news = False
 
             for ti, mlist in enumerate(models):
-
                 if tmis[ti] < len(mlist):
-
                     mname = mnames[ti][tmis[ti]]
                     isnext = True
                     for tj, jnames in enumerate(mnames):
@@ -98,7 +112,6 @@ class FarmController(FarmDataModel):
                             break
 
                     if isnext:
-
                         m = models[ti][tmis[ti]]
                         tmodels.append(m)
 
@@ -138,7 +151,7 @@ class FarmController(FarmDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         """
@@ -150,7 +163,6 @@ class FarmController(FarmDataModel):
         for ti, t in enumerate(algo.farm.turbines):
             prer = None
             for mi, mname in enumerate(t.models):
-
                 istype = False
                 if mname in algo.mbook.turbine_types:
                     m = algo.mbook.turbine_types[mname]
@@ -172,7 +184,7 @@ class FarmController(FarmDataModel):
                     raise KeyError(
                         f"Model {mname} not found in model book types or models"
                     )
-                    
+
                 if istype:
                     if self.turbine_types[ti] is None:
                         self.turbine_types[ti] = m
@@ -207,29 +219,32 @@ class FarmController(FarmDataModel):
             algo, pre_rotor=False, models=postr_models
         )
         tmsels = tmsels_pre + tmsels_post
+        self._tmall = [np.all(t) for t in tmsels]
         self.turbine_model_names = mnames_pre + mnames_post
         if len(self.turbine_model_names):
-            self.turbine_model_sels = np.stack(tmsels, axis=2)
+            self._tmsels = np.stack(tmsels, axis=2)
         else:
             raise ValueError(f"Controller '{self.name}': No turbine model found.")
 
-    def __get_pars(self, algo, models, ptype, mdata=None, st_sel=None, from_data=True):
+    def __get_pars(self, algo, models, ptype, mdata=None, downwind_index=None):
         """
         Private helper function for gathering model parameters.
         """
-        if from_data:
-            s = mdata[FV.TMODEL_SELS]
-        else:
-            s = self.turbine_model_sels
-        if st_sel is not None:
-            s = s & st_sel[:, :, None]
-
-        pars = [
-            {"st_sel": s[:, :, self.turbine_model_names.index(m.name)]} for m in models
-        ]
-        for mi, m in enumerate(models):
+        pars = []
+        for m in models:
+            mi = self.turbine_model_names.index(m.name)
+            if self._tmall[mi]:
+                s = np.s_[:, :] if downwind_index is None else np.s_[:, downwind_index]
+            else:
+                if downwind_index is None:
+                    s = mdata[FC.TMODEL_SELS][:, :, mi]
+                else:
+                    s = np.s_[
+                        mdata[FC.TMODEL_SELS][:, downwind_index, mi], downwind_index
+                    ]
+            pars.append({"st_sel": s})
             if m.name in self.pars:
-                pars[mi].update(self.pars[m.name][ptype])
+                pars[-1].update(self.pars[m.name][ptype])
 
         return pars
 
@@ -237,40 +252,48 @@ class FarmController(FarmDataModel):
         """
         Initializes the model.
 
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        self.collect_models(algo)
+        super().initialize(algo, verbosity)
+
+    def load_data(self, algo, verbosity=0):
+        """
+        Load and/or create all model data that is subject to chunking.
+
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         Returns
         -------
-        idata : dict
+        idata: dict
             The dict has exactly two entries: `data_vars`,
             a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
             and `coords`, a dict with entries `dim_name_str -> dim_array`
 
         """
-        idata = super().initialize(algo, verbosity)
+        idata = super().load_data(algo, verbosity)
 
-        self.collect_models(algo)
-
-        # done by algo.update_idata
-        #algo.update_idata([self.pre_rotor_models, self.post_rotor_models], 
-        #    idata=idata, verbosity=verbosity)
-
-        idata["coords"][FV.TMODELS] = self.turbine_model_names
-        idata["data_vars"][FV.TMODEL_SELS] = (
-            (FV.STATE, FV.TURBINE, FV.TMODELS),
-            self.turbine_model_sels,
+        idata["coords"][FC.TMODELS] = self.turbine_model_names
+        idata["data_vars"][FC.TMODEL_SELS] = (
+            (FC.STATE, FC.TURBINE, FC.TMODELS),
+            self._tmsels,
         )
+        self._tmsels = None
 
         return idata
 
@@ -280,12 +303,12 @@ class FarmController(FarmDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -296,7 +319,7 @@ class FarmController(FarmDataModel):
             )
         )
 
-    def calculate(self, algo, mdata, fdata, pre_rotor, st_sel=None):
+    def calculate(self, algo, mdata, fdata, pre_rotor, downwind_index=None):
         """ "
         The main model calculation.
 
@@ -305,30 +328,28 @@ class FarmController(FarmDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pre_rotor : bool
+        pre_rotor: bool
             Flag for running pre-rotor or post-rotor
             models
-        st_sel : numpy.ndarray of bool, optional
-            Selection of states and turbines, shape:
-            (n_states, n_turbines). None for all.
+        downwind_index: int, optional
+            The index in the downwind order
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray with shape (n_states, n_turbines)
 
         """
         s = self.pre_rotor_models if pre_rotor else self.post_rotor_models
-        pars = self.__get_pars(algo, s.models, "calc", mdata, st_sel, from_data=True)
+        pars = self.__get_pars(algo, s.models, "calc", mdata, downwind_index)
         res = s.calculate(algo, mdata, fdata, parameters=pars)
-        self.turbine_model_sels = mdata[FV.TMODEL_SELS]
         return res
 
     def finalize(self, algo, verbosity=0):
@@ -337,17 +358,11 @@ class FarmController(FarmDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 means silent
 
         """
-        for s in [self.pre_rotor_models, self.post_rotor_models]:
-            if s is not None and s.initialized:
-                algo.finalize_model(s, verbosity)
-
-        self.turbine_model_names = None
-        self.turbine_model_sels = None
-
         super().finalize(algo, verbosity)
+        self.turbine_model_names = None
