@@ -1,7 +1,8 @@
 from abc import abstractmethod
+import numpy as np
 
 from .data_calc_model import DataCalcModel
-import foxes.variables as FV
+import foxes.constants as FC
 
 
 class FarmDataModel(DataCalcModel):
@@ -9,21 +10,27 @@ class FarmDataModel(DataCalcModel):
     Abstract base class for models that modify
     farm data.
 
-    Parameters
+    Attributes
     ----------
-    pre_rotor : bool
+    pre_rotor: bool
         Flag for running this model before
         running the rotor model.
 
-    Attributes
-    ----------
-    pre_rotor : bool
-        Flag for running this model before
-        running the rotor model.
+    :group: core
 
     """
 
     def __init__(self, pre_rotor=False):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        pre_rotor: bool
+            Flag for running this model before
+            running the rotor model.
+
+        """
         super().__init__()
         self.pre_rotor = pre_rotor
 
@@ -34,17 +41,38 @@ class FarmDataModel(DataCalcModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
         return []
 
+    def ensure_variables(self, algo, mdata, fdata):
+        """
+        Add variables to fdata, initialized with NaN
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        mdata: foxes.core.Data
+            The model data
+        fdata: foxes.core.Data
+            The farm data
+
+        """
+        n_states = fdata.n_states
+        n_turbines = fdata.n_turbines
+        for v in self.output_farm_vars(algo):
+            if v not in fdata:
+                fdata[v] = np.full((n_states, n_turbines), np.nan, dtype=FC.DTYPE)
+                fdata.dims[v] = (FC.STATE, FC.TURBINE)
+
     @abstractmethod
     def calculate(self, algo, mdata, fdata):
         """ "
@@ -55,16 +83,16 @@ class FarmDataModel(DataCalcModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray with shape (n_states, n_turbines)
 
@@ -80,18 +108,18 @@ class FarmDataModel(DataCalcModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        *data : tuple of xarray.Dataset
+        *data: tuple of xarray.Dataset
             The input data
         out_vars: list of str
             The calculation output variables
-        **calc_pars : dict, optional
+        **calc_pars: dict, optional
             Additional arguments for the `calculate` function
 
         Returns
         -------
-        results : xarray.Dataset
+        results: xarray.Dataset
             The calculation results
 
         """
@@ -99,8 +127,8 @@ class FarmDataModel(DataCalcModel):
             algo,
             *data,
             out_vars=out_vars,
-            loop_dims=[FV.STATE],
-            out_core_vars=[FV.TURBINE, FV.VARS],
+            loop_dims=[FC.STATE],
+            out_core_vars=[FC.TURBINE, FC.VARS],
             **calc_pars,
         )
 
@@ -121,19 +149,25 @@ class FarmDataModelList(FarmDataModel):
     `calculate` functions are called together
     under one common call of xarray's `apply_ufunc`.
 
-    Parameters
-    ----------
-    models : list of foxes.core.FarmDataModel
-        The model list
-
     Attributes
     ----------
-    models : list of foxes.core.FarmDataModel
+    models: list of foxes.core.FarmDataModel
         The model list
 
+    :group: core
+
     """
 
     def __init__(self, models=[]):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        models: list of foxes.core.FarmDataModel
+            The model list
+
+        """
         super().__init__()
         self.models = models
 
@@ -143,24 +177,50 @@ class FarmDataModelList(FarmDataModel):
 
         Parameters
         ----------
-        model : foxes.core.FarmDataModel
+        model: foxes.core.FarmDataModel
             The model to add
 
         """
         self.models.append(model)
 
+    def insert(self, index, model):
+        """
+        Insert a model into the list
+
+        Parameters
+        ----------
+        index: int
+            The index in the model list
+        model: foxes.core.FarmDataModel
+            The model to insert
+
+        """
+        self.models.insert(index, model)
+
+    def sub_models(self):
+        """
+        List of all sub-models
+
+        Returns
+        -------
+        smdls: list of foxes.core.Model
+            Names of all sub models
+
+        """
+        return self.models
+
     def output_farm_vars(self, algo):
         """
         The variables which are being modified by the model.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -169,38 +229,6 @@ class FarmDataModelList(FarmDataModel):
             ovars += m.output_farm_vars(algo)
         return list(dict.fromkeys(ovars))
 
-    def initialize(self, algo, verbosity=0):
-        """
-        Initializes the model.
-
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level, 0 = silent
-
-        Returns
-        -------
-        idata : dict
-            The dict has exactly two entries: `data_vars`,
-            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
-            and `coords`, a dict with entries `dim_name_str -> dim_array`
-
-        """
-        idata = super().initialize(algo, verbosity)
-
-        # done by algo.update_idata
-        #algo.update_idata(self.models, idata=idata, verbosity=verbosity)
-
-        return idata
-
     def calculate(self, algo, mdata, fdata, parameters=[]):
         """
         The main model calculation.
@@ -210,18 +238,18 @@ class FarmDataModelList(FarmDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        parameters : list of dict, optional
+        parameters: list of dict, optional
             A list of parameter dicts, one for each model
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray with shape (n_states, n_turbines)
 
@@ -243,21 +271,3 @@ class FarmDataModelList(FarmDataModel):
             fdata.update(res)
 
         return {v: fdata[v] for v in self.output_farm_vars(algo)}
-
-    def finalize(self, algo, verbosity=0):
-        """
-        Finalizes the model.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level, 0 means silent
-
-        """
-        for m in self.models:
-            if m.initialized:
-                algo.finalize_model(m, verbosity)
-
-        super().finalize(algo, verbosity)
