@@ -1,13 +1,17 @@
+import numpy as np
 from abc import abstractmethod
 
 from .data_calc_model import DataCalcModel
-import foxes.variables as FV
+import foxes.constants as FC
 
 
 class PointDataModel(DataCalcModel):
     """
     Abstract base class for models that modify
     point based data.
+
+    :group: core
+
     """
 
     @abstractmethod
@@ -15,21 +19,41 @@ class PointDataModel(DataCalcModel):
         """
         The variables which are being modified by the model.
 
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
         return []
 
+    def ensure_variables(self, algo, mdata, fdata, tdata):
+        """
+        Add variables to tdata, initialized with NaN
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        mdata: foxes.core.Data
+            The model data
+        fdata: foxes.core.Data
+            The farm data
+        tdata: foxes.core.Data
+            The target point data
+
+        """
+        for v in self.output_point_vars(algo):
+            if v not in tdata:
+                tdata[v] = np.full(
+                    (tdata.n_states, tdata.n_targets, tdata.n_tpoints),
+                    np.nan,
+                    dtype=FC.DTYPE,
+                )
+                tdata.dims[v] = (FC.STATE, FC.TARGET, FC.TPOINT)
+
     @abstractmethod
-    def calculate(self, algo, mdata, fdata, pdata):
+    def calculate(self, algo, mdata, fdata, tdata):
         """ "
         The main model calculation.
 
@@ -38,20 +62,21 @@ class PointDataModel(DataCalcModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pdata : foxes.core.Data
-            The point data
+        tdata: foxes.core.TData
+            The target point data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
-            Values: numpy.ndarray with shape (n_states, n_points)
+            Values: numpy.ndarray with shape
+            (n_states, n_targets, n_tpoints)
 
         """
         pass
@@ -65,29 +90,34 @@ class PointDataModel(DataCalcModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        *data : tuple of xarray.Dataset
+        *data: tuple of xarray.Dataset
             The input data
         out_vars: list of str
             The calculation output variables
-        **calc_pars : dict, optional
+        **calc_pars: dict, optional
             Additional arguments for the `calculate` function
 
         Returns
         -------
-        results : xarray.Dataset
+        results: xarray.Dataset
             The calculation results
 
         """
-        return super().run_calculation(
+        results = super().run_calculation(
             algo,
             *data,
             out_vars=out_vars,
-            loop_dims=[FV.STATE, FV.POINT],
-            out_core_vars=[FV.VARS],
+            loop_dims=[FC.STATE, FC.TARGET],
+            out_core_vars=[FC.TPOINT, FC.VARS],
             **calc_pars,
         )
+        if results.sizes[FC.TPOINT] != 1:
+            raise ValueError(
+                f"PointDataModel '{self.name}': Expecting dimension '{FC.TPOINT}' of size 1, found {results.sizes[FC.TPOINT]}"
+            )
+        return results.sel({FC.TPOINT: 0}).rename({FC.TARGET: FC.POINT})
 
     def __add__(self, m):
         if isinstance(m, list):
@@ -106,19 +136,25 @@ class PointDataModelList(PointDataModel):
     `calculate` functions are called together
     under one common call of xarray's `apply_ufunc`.
 
-    Parameters
-    ----------
-    models : list of foxes.core.PointDataModel
-        The model list
-
     Attributes
     ----------
-    models : list of foxes.core.PointDataModel
+    models: list of foxes.core.PointDataModel
         The model list
 
+    :group: core
+
     """
 
     def __init__(self, models=[]):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        models: list of foxes.core.PointDataModel
+            The model list
+
+        """
         super().__init__()
         self.models = models
 
@@ -128,24 +164,36 @@ class PointDataModelList(PointDataModel):
 
         Parameters
         ----------
-        model : foxes.core.PointDataModel
+        model: foxes.core.PointDataModel
             The model to add
 
         """
         self.models.append(model)
 
+    def sub_models(self):
+        """
+        List of all sub-models
+
+        Returns
+        -------
+        smdls: list of foxes.core.Model
+            Names of all sub models
+
+        """
+        return self.models
+
     def output_point_vars(self, algo):
         """
         The variables which are being modified by the model.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -154,43 +202,7 @@ class PointDataModelList(PointDataModel):
             ovars += m.output_point_vars(algo)
         return list(dict.fromkeys(ovars))
 
-    def initialize(self, algo, verbosity=0):
-        """
-        Initializes the model.
-
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level, 0 = silent
-
-        Returns
-        -------
-        idata : dict
-            The dict has exactly two entries: `data_vars`,
-            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
-            and `coords`, a dict with entries `dim_name_str -> dim_array`
-
-        """
-        if verbosity > 1:
-            print(f"-- {self.name}: Starting initialization -- ")
-
-        idata = super().initialize(algo)
-        algo.update_idata(self.models, idata=idata, verbosity=verbosity)
-
-        if verbosity > 1:
-            print(f"-- {self.name}: Finished initialization -- ")
-
-        return idata
-
-    def calculate(self, algo, mdata, fdata, pdata, parameters=None):
+    def calculate(self, algo, mdata, fdata, tdata, parameters=None):
         """ "
         The main model calculation.
 
@@ -199,22 +211,23 @@ class PointDataModelList(PointDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pdata : foxes.core.Data
-            The point data
-        parameters : list of dict, optional
+        tdata: foxes.core.TData
+            The target point data
+        parameters: list of dict, optional
             A list of parameter dicts, one for each model
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
-            Values: numpy.ndarray with shape (n_states, n_points)
+            Values: numpy.ndarray with shape
+            (n_states, n_targets, n_tpoints)
 
         """
         if parameters is None:
@@ -229,25 +242,7 @@ class PointDataModelList(PointDataModel):
             )
 
         for mi, m in enumerate(self.models):
-            res = m.calculate(algo, mdata, fdata, pdata, **parameters[mi])
-            pdata.update(res)
+            res = m.calculate(algo, mdata, fdata, tdata, **parameters[mi])
+            tdata.update(res)
 
-        return {v: pdata[v] for v in self.output_point_vars(algo)}
-
-    def finalize(self, algo, verbosity=0):
-        """
-        Finalizes the model.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level, 0 means silent
-
-        """
-        for m in self.models:
-            if m.initialized:
-                algo.finalize_model(m, verbosity)
-
-        super().finalize(algo, verbosity)
+        return {v: tdata[v] for v in self.output_point_vars(algo)}
