@@ -4,8 +4,9 @@ from abc import abstractmethod
 import foxes.variables as FV
 import foxes.constants as FC
 from .farm_data_model import FarmDataModel
-from .data import Data
-from foxes.utils import wd2uv, uv2wd
+from foxes.utils import wd2uv, uv2wd, all_subclasses
+
+from .data import TData
 
 
 class RotorModel(FarmDataModel):
@@ -16,42 +17,55 @@ class RotorModel(FarmDataModel):
     states, and provide rotor points and weights
     for the calculation of rotor effective quantities.
 
-    Parameters
+    Attributes
     ----------
-    calc_vars : list of str
+    calc_vars: list of str
         The variables that are calculated by the model
         (Their ambients are added automatically)
 
-    Attributes
-    ----------
-    calc_vars : list of str
-        The variables that are calculated by the model
-        (Their ambients are added automatically)
+    :group: core
 
     """
 
     def __init__(self, calc_vars):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        calc_vars: list of str
+            The variables that are calculated by the model
+            (Their ambients are added automatically)
+
+        """
         super().__init__()
         self.calc_vars = calc_vars
 
+        self.RPOINTS = self.var("rpoints")
+        self.RWEIGHTS = self.var("rweights")
+        self.AMBRES = self.var("amb_res")
+
     def output_farm_vars(self, algo):
         """
         The variables which are being modified by the model.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
-        return self.calc_vars + [
-            FV.var2amb[v] for v in self.calc_vars if v in FV.var2amb
-        ]
+        return list(
+            set(
+                self.calc_vars
+                + [FV.var2amb[v] for v in self.calc_vars if v in FV.var2amb]
+            )
+        )
 
     @abstractmethod
     def n_rotor_points(self):
@@ -60,7 +74,7 @@ class RotorModel(FarmDataModel):
 
         Returns
         -------
-        n_rpoints : int
+        n_rpoints: int
             The number of rotor points
 
         """
@@ -73,7 +87,7 @@ class RotorModel(FarmDataModel):
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights of the rotor points,
             add to one, shape: (n_rpoints,)
 
@@ -94,7 +108,7 @@ class RotorModel(FarmDataModel):
 
         Returns
         -------
-        dpoints : numpy.ndarray
+        dpoints: numpy.ndarray
             The design points, shape: (n_points, 3)
 
         """
@@ -106,16 +120,16 @@ class RotorModel(FarmDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
 
         Returns
         -------
-        points : numpy.ndarray
+        points: numpy.ndarray
             The rotor points, shape:
             (n_states, n_turbines, n_rpoints, 3)
 
@@ -142,17 +156,17 @@ class RotorModel(FarmDataModel):
 
         return points
 
-    def _set_res(self, fdata, v, res, stsel):
+    def _set_res(self, fdata, v, res, downwind_index):
         """
         Helper function for results setting
         """
-        if stsel is None:
+        if downwind_index is None:
             fdata[v] = res.copy()
         elif res.shape[1] == 1:
-            fdata[v][stsel] = res[:, 0]
+            fdata[v][:, downwind_index] = res[:, 0]
         else:
             raise ValueError(
-                f"Rotor model '{self.name}': states_turbine is not None, but results shape for '{v}' has more than one turbine, {res.shape}"
+                f"Rotor model '{self.name}': downwind_index is not None, but results shape for '{v}' has more than one turbine, {res.shape}"
             )
 
     def eval_rpoint_results(
@@ -160,9 +174,9 @@ class RotorModel(FarmDataModel):
         algo,
         mdata,
         fdata,
-        rpoint_results,
+        tdata,
         weights,
-        states_turbine=None,
+        downwind_index=None,
         copy_to_ambient=False,
     ):
         """
@@ -176,34 +190,23 @@ class RotorModel(FarmDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        rpoint_results : dict
-            The results at rotor points. Keys: variable str.
-            Values: numpy.ndarray, shape if `states_turbine`
-            is None: (n_states, n_turbines, n_rpoints).
-            Else: (n_states, 1, n_rpoints)
-        weights : numpy.ndarray
+        tdata: foxes.core.TData
+            The target point data
+        weights: numpy.ndarray
             The rotor point weights, shape: (n_rpoints,)
-        states_turbine: numpy.ndarray of int, optional
-            The turbine indices, one per state. Shape: (n_states,)
-        copy_to_ambient : bool, optional
+        downwind_index: int, optional
+            The index in the downwind order
+        copy_to_ambient: bool
             If `True`, the fdata results are copied to ambient
             variables after calculation
 
         """
-
-        n_states = mdata.n_states
-        n_turbines = algo.n_turbines
-        if states_turbine is not None:
-            stsel = (np.arange(n_states), states_turbine)
-        else:
-            stsel = None
-
         uvp = None
         uv = None
         if (
@@ -214,28 +217,23 @@ class RotorModel(FarmDataModel):
             or FV.REWS2 in self.calc_vars
             or FV.REWS3 in self.calc_vars
         ):
-
-            wd = rpoint_results[FV.WD]
-            ws = rpoint_results[FV.WS]
+            wd = tdata[FV.WD]
+            ws = tdata[FV.WS]
             uvp = wd2uv(wd, ws, axis=-1)
             uv = np.einsum("stpd,p->std", uvp, weights)
 
         wd = None
         vdone = []
         for v in self.calc_vars:
-
-            if v not in fdata:
-                fdata[v] = np.zeros((n_states, n_turbines), dtype=FC.DTYPE)
-
             if v == FV.WD or v == FV.YAW:
                 if wd is None:
                     wd = uv2wd(uv, axis=-1)
-                self._set_res(fdata, v, wd, stsel)
+                self._set_res(fdata, v, wd, downwind_index)
                 vdone.append(v)
 
             elif v == FV.WS:
                 ws = np.linalg.norm(uv, axis=-1)
-                self._set_res(fdata, v, ws, stsel)
+                self._set_res(fdata, v, ws, downwind_index)
                 del ws
                 vdone.append(v)
         del uv, wd
@@ -245,24 +243,21 @@ class RotorModel(FarmDataModel):
             or FV.REWS2 in self.calc_vars
             or FV.REWS3 in self.calc_vars
         ):
-
-            if stsel is None:
+            if downwind_index is None:
                 yaw = fdata[FV.YAW].copy()
             else:
-                yaw = fdata[FV.YAW][stsel][:, None]
+                yaw = fdata[FV.YAW][:, downwind_index, None]
             nax = wd2uv(yaw, axis=-1)
             wsp = np.einsum("stpd,std->stp", uvp, nax)
 
             for v in self.calc_vars:
-
                 if v == FV.REWS:
                     rews = np.maximum(np.einsum("stp,p->st", wsp, weights), 0.0)
-                    self._set_res(fdata, v, rews, stsel)
+                    self._set_res(fdata, v, rews, downwind_index)
                     del rews
                     vdone.append(v)
 
                 elif v == FV.REWS2:
-
                     # For highly inhomogeneous wind fields
                     # and multiple rotor points some of the uv
                     # vectors may have negative projections onto the
@@ -270,20 +265,17 @@ class RotorModel(FarmDataModel):
                     if uvp.shape[2] > 1:
                         rews2 = np.sqrt(
                             np.maximum(
-                                np.einsum(
-                                    "stp,p->st", np.sign(wsp) * wsp**2, weights
-                                ),
+                                np.einsum("stp,p->st", np.sign(wsp) * wsp**2, weights),
                                 0.0,
                             )
                         )
                     else:
                         rews2 = np.sqrt(np.einsum("stp,p->st", wsp**2, weights))
-                    self._set_res(fdata, v, rews2, stsel)
+                    self._set_res(fdata, v, rews2, downwind_index)
                     del rews2
                     vdone.append(v)
 
                 elif v == FV.REWS3:
-
                     # For highly inhomogeneous wind fields
                     # and multiple rotor points some of the uv
                     # vectors may have negative projections onto the
@@ -293,10 +285,8 @@ class RotorModel(FarmDataModel):
                             np.einsum("stp,p->st", wsp**3, weights), 0.0
                         ) ** (1.0 / 3.0)
                     else:
-                        rews3 = (np.einsum("stp,p->st", wsp**3, weights)) ** (
-                            1.0 / 3.0
-                        )
-                    self._set_res(fdata, v, rews3, stsel)
+                        rews3 = (np.einsum("stp,p->st", wsp**3, weights)) ** (1.0 / 3.0)
+                    self._set_res(fdata, v, rews3, downwind_index)
                     del rews3
                     vdone.append(v)
 
@@ -305,8 +295,8 @@ class RotorModel(FarmDataModel):
 
         for v in self.calc_vars:
             if v not in vdone:
-                res = np.einsum("stp,p->st", rpoint_results[v], weights)
-                self._set_res(fdata, v, res, stsel)
+                res = np.einsum("stp,p->st", tdata[v], weights)
+                self._set_res(fdata, v, res, downwind_index)
             if copy_to_ambient and v in FV.var2amb:
                 fdata[FV.var2amb[v]] = fdata[v].copy()
 
@@ -320,92 +310,117 @@ class RotorModel(FarmDataModel):
         store_rpoints=False,
         store_rweights=False,
         store_amb_res=False,
-        states_turbine=None,
+        downwind_index=None,
     ):
         """
         Calculate ambient rotor effective results.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        rpoints : numpy.ndarray, optional
+        rpoints: numpy.ndarray, optional
             The rotor points, or None for automatic for
             this rotor. Shape: (n_states, n_turbines, n_rpoints, 3)
-        weights : numpy.ndarray, optional
+        weights: numpy.ndarray, optional
             The rotor point weights, or None for automatic
             for this rotor. Shape: (n_rpoints,)
-        store_rpoints : bool, optional
+        store_rpoints: bool, optional
             Switch for storing rotor points to mdata
-        store_rweights : bool, optional
+        store_rweights: bool, optional
             Switch for storing rotor point weights to mdata
-        store_amb_res : bool, optional
+        store_amb_res: bool, optional
             Switch for storing ambient rotor point reults as they
             come from the states to mdata
-        states_turbine: numpy.ndarray of int, optional
-            The turbine indices, one per state. Shape: (n_states,)
+        downwind_index: int, optional
+            Only compute for index in the downwind order
 
         Returns
         -------
-        results : dict
+        results: dict
             results dict. Keys: Variable name str. Values:
             numpy.ndarray with results, shape: (n_states, n_turbines)
 
         """
 
         if rpoints is None:
-            rpoints = mdata.get(FV.RPOINTS, self.get_rotor_points(algo, mdata, fdata))
-        if states_turbine is not None:
-            n_states = mdata.n_states
-            stsel = (np.arange(n_states), states_turbine)
-            rpoints = rpoints[stsel][:, None]
-        n_states, n_turbines, n_rpoints, __ = rpoints.shape
-        n_points = n_turbines * n_rpoints
+            rpoints = mdata.get(self.RPOINTS, self.get_rotor_points(algo, mdata, fdata))
+        if store_rpoints:
+            mdata[self.RPOINTS] = rpoints
+            mdata.dims[self.RPOINTS] = (FC.STATE, FC.TURBINE, FC.TPOINT, FC.XYH)
+            self.data_to_store(self.RPOINTS, algo, mdata)
+
+        if downwind_index is not None:
+            rpoints = rpoints[:, downwind_index, None]
 
         if weights is None:
-            weights = mdata.get(FV.RWEIGHTS, self.rotor_point_weights())
-
-        if store_rpoints:
-            mdata[FV.RPOINTS] = rpoints
-            mdata.dims[FV.RPOINTS] = (FV.STATE, FV.TURBINE, FV.RPOINT, FV.XYH)
+            weights = mdata.get(FC.TWEIGHTS, self.rotor_point_weights())
         if store_rweights:
-            mdata[FV.RWEIGHTS] = weights
-            mdata.dims[FV.RWEIGHTS] = (FV.RPOINT,)
+            mdata[self.RWEIGHTS] = weights
+            mdata.dims[self.RWEIGHTS] = (FC.TPOINT,)
+            self.data_to_store(self.RWEIGHTS, algo, mdata)
 
+        tdata = TData.from_tpoints(rpoints, weights)
         svars = algo.states.output_point_vars(algo)
-        points = rpoints.reshape(n_states, n_points, 3)
-        pdata = {FV.POINTS: points}
-        pdims = {FV.POINTS: (FV.STATE, FV.POINT, FV.XYH)}
-        pdata.update(
-            {v: np.full((n_states, n_points), np.nan, dtype=FC.DTYPE) for v in svars}
-        )
-        pdims.update({v: (FV.STATE, FV.POINT) for v in svars})
-        pdata = Data(pdata, pdims, loop_dims=[FV.STATE, FV.POINT])
-        del pdims, points
-
-        sres = algo.states.calculate(algo, mdata, fdata, pdata)
-        pdata.update(sres)
-        del sres
-
-        rpoint_results = {}
         for v in svars:
-            rpoint_results[v] = pdata[v].reshape(n_states, n_turbines, n_rpoints)
+            tdata.add(
+                v,
+                data=np.full_like(rpoints[..., 0], np.nan),
+                dims=(FC.STATE, FC.TARGET, FC.TPOINT),
+            )
+
+        sres = algo.states.calculate(algo, mdata, fdata, tdata)
+        tdata.update(sres)
 
         if store_amb_res:
-            mdata[FV.AMB_RPOINT_RESULTS] = rpoint_results
+            mdata[self.AMBRES] = sres.copy()
+            self.data_to_store(self.AMBRES, algo, mdata)
 
         self.eval_rpoint_results(
             algo,
             mdata,
             fdata,
-            rpoint_results,
+            tdata,
             weights,
-            states_turbine,
+            downwind_index,
             copy_to_ambient=True,
         )
 
         return {v: fdata[v] for v in self.output_farm_vars(algo)}
+
+    @classmethod
+    def new(cls, rmodel_type, *args, **kwargs):
+        """
+        Run-time rotor model factory.
+
+        Parameters
+        ----------
+        rmodel_type: str
+            The selected derived class name
+        args: tuple, optional
+            Additional parameters for constructor
+        kwargs: dict, optional
+            Additional parameters for constructor
+
+        """
+
+        if rmodel_type is None:
+            return None
+
+        allc = all_subclasses(cls)
+        found = rmodel_type in [scls.__name__ for scls in allc]
+
+        if found:
+            for scls in allc:
+                if scls.__name__ == rmodel_type:
+                    return scls(*args, **kwargs)
+
+        else:
+            estr = "Rotor model type '{}' is not defined, available types are \n {}".format(
+                rmodel_type, sorted([i.__name__ for i in allc])
+            )
+            raise KeyError(estr)
