@@ -1,7 +1,9 @@
 from abc import abstractmethod
 
 from .point_data_model import PointDataModel, PointDataModelList
+from foxes.utils import all_subclasses
 import foxes.variables as FV
+import foxes.constants as FC
 
 
 class States(PointDataModel):
@@ -12,6 +14,8 @@ class States(PointDataModel):
     typically wind speed, wind direction, turbulence
     intensity and air density.
 
+    :group: core
+
     """
 
     @abstractmethod
@@ -33,7 +37,7 @@ class States(PointDataModel):
 
         Returns
         -------
-        indices : array_like
+        indices: array_like
             The index labels of states, or None for default integers
 
         """
@@ -46,48 +50,56 @@ class States(PointDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights, shape: (n_states, n_turbines)
 
         """
         pass
 
-    def _update_idata(self, algo, idata):
+    def load_data(self, algo, verbosity=0):
         """
-        Add basic states data to idata.
+        Load and/or create all model data that is subject to chunking.
 
-        This function should be called within initialize,
-        after loading the data.
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        idata : dict
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        Returns
+        -------
+        idata: dict
             The dict has exactly two entries: `data_vars`,
             a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
             and `coords`, a dict with entries `dim_name_str -> dim_array`
 
         """
+        idata = super().load_data(algo, verbosity)
+
         sinds = self.index()
         if sinds is not None:
-            idata["coords"][FV.STATE] = sinds
+            idata["coords"][FC.STATE] = sinds
 
         weights = self.weights(algo)
         if len(weights.shape) != 2:
             raise ValueError(
-                f"States '{self.name}': Wrong weights dimension, expecing ({FV.STATE}, {FV.TURBINE}), got shape {weights.shape}"
+                f"States '{self.name}': Wrong weights dimension, expecing ({FC.STATE}, {FC.TURBINE}), got shape {weights.shape}"
             )
         if weights.shape[1] != algo.n_turbines:
             raise ValueError(
-                f"States '{self.name}': Wrong size of second axis dimension '{FV.TURBINE}': Expecting {algo.n_turbines}, got {weights.shape[1]}"
+                f"States '{self.name}': Wrong size of second axis dimension '{FC.TURBINE}': Expecting {algo.n_turbines}, got {weights.shape[1]}"
             )
-        idata["data_vars"][FV.WEIGHT] = ((FV.STATE, FV.TURBINE), weights)
+        idata["data_vars"][FV.WEIGHT] = ((FC.STATE, FC.TURBINE), weights)
 
         return idata
 
@@ -97,17 +109,17 @@ class States(PointDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
         return [FV.WS, FV.WD, FV.TI, FV.RHO]
-    
+
     def __add__(self, s):
         if isinstance(s, list):
             return ExtendedStates(self, s)
@@ -120,28 +132,66 @@ class States(PointDataModel):
         else:
             return ExtendedStates(self, [s])
 
+    @classmethod
+    def new(cls, states_type, *args, **kwargs):
+        """
+        Run-time states factory.
+
+        Parameters
+        ----------
+        states_type: str
+            The selected derived class name
+        args: tuple, optional
+            Additional parameters for constructor
+        kwargs: dict, optional
+            Additional parameters for constructor
+
+        """
+
+        if states_type is None:
+            return None
+
+        allc = all_subclasses(cls)
+        found = states_type in [scls.__name__ for scls in allc]
+
+        if found:
+            for scls in allc:
+                if scls.__name__ == states_type:
+                    return scls(*args, **kwargs)
+        else:
+            estr = "States type '{}' is not defined, available types are \n {}".format(
+                states_type, sorted([i.__name__ for i in allc])
+            )
+            raise KeyError(estr)
+
 
 class ExtendedStates(States):
     """
     States extended by point data models.
 
-    Parameters
-    ----------
-    states : foxes.core.States
-        The base states to start from
-    point_models : list of foxes.core.PointDataModel, optional
-        The point models, executed after states
-
     Attributes
     ----------
-    states : foxes.core.States
+    states: foxes.core.States
         The base states to start from
-    pmodels : foxes.core.PointDataModelList
+    pmodels: foxes.core.PointDataModelList
         The point models, including states as first model
 
+    :group: core
+
     """
 
     def __init__(self, states, point_models=[]):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        states: foxes.core.States
+            The base states to start from
+        point_models: list of foxes.core.PointDataModel, optional
+            The point models, executed after states
+
+        """
         super().__init__()
         self.states = states
         self.pmodels = PointDataModelList(models=[states] + point_models)
@@ -152,12 +202,24 @@ class ExtendedStates(States):
 
         Parameters
         ----------
-        model : foxes.core.PointDataModel
+        model: foxes.core.PointDataModel
             The model to add
 
         """
         self.pmodels.append(model)
 
+    def sub_models(self):
+        """
+        List of all sub-models
+
+        Returns
+        -------
+        smdls: list of foxes.core.Model
+            Names of all sub models
+
+        """
+        return [self.pmodels]
+
     def size(self):
         """
         The total number of states.
@@ -176,7 +238,7 @@ class ExtendedStates(States):
 
         Returns
         -------
-        indices : array_like
+        indices: array_like
             The index labels of states, or None for default integers
 
         """
@@ -188,65 +250,35 @@ class ExtendedStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights, shape: (n_states, n_turbines)
 
         """
         return self.states.weights(algo)
 
-    def initialize(self, algo, verbosity=0):
-        """
-        Initializes the model.
-
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level, 0 = silent
-
-        Returns
-        -------
-        idata : dict
-            The dict has exactly two entries: `data_vars`,
-            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
-            and `coords`, a dict with entries `dim_name_str -> dim_array`
-
-        """
-        idata = super().initialize(algo, verbosity)
-        algo.update_idata(self.pmodels, idata=idata, verbosity=verbosity)
-
-        return idata
-
     def output_point_vars(self, algo):
         """
         The variables which are being modified by the model.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
         return self.states.output_point_vars(algo)
 
-    def calculate(self, algo, mdata, fdata, pdata):
+    def calculate(self, algo, mdata, fdata, tdata):
         """ "
         The main model calculation.
 
@@ -255,38 +287,23 @@ class ExtendedStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pdata : foxes.core.Data
-            The point data
+        tdata: foxes.core.TData
+            The target point data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray with shape (n_states, n_points)
 
         """
-        return self.pmodels.calculate(algo, mdata, fdata, pdata)
-
-    def finalize(self, algo, verbosity=0):
-        """
-        Finalizes the model.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level
-
-        """
-        self.pmodels.finalize(algo,  verbosity)
-        super().finalize(algo, verbosity)
+        return self.pmodels.calculate(algo, mdata, fdata, tdata)
 
     def __add__(self, m):
         models = self.pmodels.models[1:]
