@@ -1,5 +1,6 @@
 import numpy as np
 import pandas as pd
+from xarray import Dataset, open_dataset
 from pathlib import Path
 from scipy.interpolate import interp1d
 
@@ -22,50 +23,30 @@ class MultiHeightStates(States):
 
     WS-50, WS-60, WS-100, ...
 
-    Parameters
-    ----------
-    data_source : str or pandas.DataFrame
-        Either path to a file or data
-    output_vars : list of str
-        The output variables
-    heights : list of float
-        The heights at which to search data
-    var2col : dict, optional
-        Mapping from variable names to data column names
-    fixed_vars : dict, optional
-        Fixed uniform variable values, instead of
-        reading from data
-    pd_read_pars : dict, optional
-        pandas file reading parameters
-    states_sel : slice or range or list of int, optional
-        States subset selection
-    states_loc : list, optional
-        State index selection via pandas loc function
-    ipars : dict, optional
-        Parameters for scipy.interpolate.interp1d
-
     Attributes
     ----------
-    data_source : str or pandas.DataFrame
+    data_source: str or pandas.DataFrame
         Either path to a file or data
-    ovars : list of str
+    ovars: list of str
         The output variables
-    heights : list of float
+    heights: list of float
         The heights at which to search data
-    var2col : dict, optional
+    var2col: dict, optional
         Mapping from variable names to data column names
-    fixed_vars : dict, optional
+    fixed_vars: dict, optional
         Fixed uniform variable values, instead of
         reading from data
-    pd_read_pars : dict, optional
+    pd_read_pars: dict, optional
         pandas file reading parameters
-    states_sel : slice or range or list of int
+    states_sel: slice or range or list of int
         States subset selection
-    states_loc : list
+    states_loc: list
         State index selection via pandas loc function
-    RDICT : dict
+    RDICT: dict
         Default pandas file reading parameters
 
+    :group: input.states
+
     """
 
     RDICT = {"index_col": 0}
@@ -82,6 +63,32 @@ class MultiHeightStates(States):
         states_loc=None,
         ipars={},
     ):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        data_source: str or pandas.DataFrame
+            Either path to a file or data
+        output_vars: list of str
+            The output variables
+        heights: list of float
+            The heights at which to search data
+        var2col: dict, optional
+            Mapping from variable names to data column names
+        fixed_vars: dict, optional
+            Fixed uniform variable values, instead of
+            reading from data
+        pd_read_pars: dict, optional
+            pandas file reading parameters
+        states_sel: slice or range or list of int, optional
+            States subset selection
+        states_loc: list, optional
+            State index selection via pandas loc function
+        ipars: dict, optional
+            Parameters for scipy.interpolate.interp1d
+
+        """
         super().__init__()
 
         self.data_source = data_source
@@ -94,7 +101,6 @@ class MultiHeightStates(States):
         self.states_sel = states_sel
         self.states_loc = states_loc
 
-        self._cmap = None
         self._solo = None
         self._weights = None
         self._N = None
@@ -105,13 +111,13 @@ class MultiHeightStates(States):
 
         Parameters
         ----------
-        states_sel : slice or range or list of int, optional
+        states_sel: slice or range or list of int, optional
             States subset selection
-        states_loc : list, optional
+        states_loc: list, optional
             State index selection via pandas loc function
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
-            
+
         """
         if self.initialized:
             if algo is None:
@@ -145,26 +151,24 @@ class MultiHeightStates(States):
                     )
             return cls
 
-    def initialize(self, algo, verbosity=0):
+    def load_data(self, algo, verbosity=0):
         """
-        Initializes the model.
+        Load and/or create all model data that is subject to chunking.
 
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         Returns
         -------
-        idata : dict
+        idata: dict
             The dict has exactly two entries: `data_vars`,
             a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
             and `coords`, a dict with entries `dim_name_str -> dim_array`
@@ -188,13 +192,13 @@ class MultiHeightStates(States):
             isorg = False
         else:
             isorg = True
+            data = self.data_source
 
         if self.states_sel is not None:
             data = data.iloc[self.states_sel]
         elif self.states_loc is not None:
             data = data.loc[self.states_loc]
-        else:
-            data = data
+
         self._N = len(data.index)
         self._inds = data.index.to_numpy()
 
@@ -213,14 +217,14 @@ class MultiHeightStates(States):
             data[col_w] = self._weights[:, 0]
 
         cols = []
-        self._cmap = {}
+        cmap = {}
         self._solo = {}
         for v in self.ovars:
             vcols = self._find_cols(v, data.columns)
             if len(vcols) == 1:
                 self._solo[v] = data[vcols[0]].to_numpy()
             elif len(vcols) > 1:
-                self._cmap[v] = (len(cols), len(cols) + len(vcols))
+                cmap[v] = (len(cols), len(cols) + len(vcols))
                 cols += vcols
         data = data[cols]
 
@@ -228,22 +232,22 @@ class MultiHeightStates(States):
         self.VARS = self.var("vars")
         self.DATA = self.var("data")
 
-        idata = super().initialize(algo, verbosity)
-        self._update_idata(algo, idata)
+        idata = super().load_data(algo, verbosity)
 
         idata["coords"][self.H] = self.heights
-        idata["coords"][self.VARS] = list(self._cmap.keys())
+        idata["coords"][self.VARS] = list(cmap.keys())
 
         n_hts = len(self.heights)
         n_vrs = int(len(data.columns) / n_hts)
-        dims = (FV.STATE, self.VARS, self.H)
+        dims = (FC.STATE, self.VARS, self.H)
         idata["data_vars"][self.DATA] = (
             dims,
             data.to_numpy().reshape(self._N, n_vrs, n_hts),
         )
 
         for v, d in self._solo.items():
-            idata["data_vars"][self.var(v)] = ((FV.STATE,), d)
+            idata["data_vars"][self.var(v)] = ((FC.STATE,), d)
+        self._solo = list(self._solo.keys())
 
         return idata
 
@@ -265,24 +269,24 @@ class MultiHeightStates(States):
 
         Returns
         -------
-        indices : array_like
+        indices: array_like
             The index labels of states, or None for default integers
 
         """
         return self._inds
-    
+
     def output_point_vars(self, algo):
         """
         The variables which are being modified by the model.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -294,18 +298,18 @@ class MultiHeightStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights, shape: (n_states, n_turbines)
 
         """
         return self._weights
 
-    def calculate(self, algo, mdata, fdata, pdata):
+    def calculate(self, algo, mdata, fdata, tdata):
         """ "
         The main model calculation.
 
@@ -314,26 +318,31 @@ class MultiHeightStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pdata : foxes.core.Data
-            The point data
+        tdata: foxes.core.TData
+            The target point data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
-            Values: numpy.ndarray with shape (n_states, n_points)
+            Values: numpy.ndarray with shape
+            (n_states, n_targets, n_tpoints)
 
         """
+        n_states = tdata.n_states
+        n_targets = tdata.n_targets
+        n_tpoints = tdata.n_tpoints
         h = mdata[self.H]
-        z = pdata[FV.POINTS][:, :, 2]
+        z = tdata[FC.TARGETS][..., 2].reshape(n_states, n_targets * n_tpoints)
         n_h = len(h)
         vrs = list(mdata[self.VARS])
+        n_vars = len(vrs)
 
         coeffs = np.zeros((n_h, n_h), dtype=FC.DTYPE)
         np.fill_diagonal(coeffs, 1.0)
@@ -361,24 +370,29 @@ class MultiHeightStates(States):
                 raise KeyError(
                     f"States '{self.name}': Found variable '{FV.WD}', but missing variable '{FV.WS}'"
                 )
-            uv = np.einsum("shd,sph->spd", uvh, ires)
+            uv = np.einsum("shd,sph->spd", uvh, ires).reshape(
+                n_states, n_targets, n_tpoints, 2
+            )
             del uvh
 
-        ires = np.einsum("svh,sph->svp", mdata[self.DATA], ires)
+        ires = np.einsum("svh,sph->vsp", mdata[self.DATA], ires).reshape(
+            n_vars, n_states, n_targets, n_tpoints
+        )
 
         results = {}
         for v in self.ovars:
-            results[v] = pdata[v]
             if has_wd and v == FV.WD:
                 results[v] = uv2wd(uv, axis=-1)
             elif has_wd and v == FV.WS:
                 results[v] = np.linalg.norm(uv, axis=-1)
             elif v in self.fixed_vars:
+                results[v] = np.zeros((n_states, n_targets, n_tpoints), dtype=FC.DTYPE)
                 results[v][:] = self.fixed_vars[v]
-            elif v in self._solo.keys():
-                results[v][:] = mdata[self.var(v)][:, None]
+            elif v in self._solo:
+                results[v] = np.zeros((n_states, n_targets, n_tpoints), dtype=FC.DTYPE)
+                results[v][:] = mdata[self.var(v)][:, None, None]
             else:
-                results[v] = ires[:, vrs.index(v)]
+                results[v] = ires[vrs.index(v)]
 
         return results
 
@@ -388,22 +402,243 @@ class MultiHeightStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level
 
         """
-        self._cmap = None
+        super().finalize(algo, verbosity)
         self._solo = None
         self._weights = None
         self._N = None
-        super().finalize(algo, verbosity)
+
+
+class MultiHeightNCStates(MultiHeightStates):
+    """
+    Multi-height states from xarray Dataset.
+
+    Attributes
+    ----------
+    data_source: str or xarray.Dataset
+        Either path to a file or data
+    state_coord: str
+        Name of the state coordinate
+    h_coord: str
+        Name of the height coordinate
+    xr_read_pars: dict
+        Parameters for xarray.open_dataset
+
+    :group: input.states
+
+    """
+
+    def __init__(
+        self,
+        data_source,
+        *args,
+        state_coord=FC.STATE,
+        h_coord=FV.H,
+        heights=None,
+        format_times_func="default",
+        xr_read_pars={},
+        **kwargs,
+    ):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        data_source: str or pandas.DataFrame
+            Either path to a file or data
+        args: tuple, optional
+            Parameters for the base class
+        state_coord: str
+            Name of the state coordinate
+        h_coord: str
+            Name of the height coordinate
+        output_vars: list of str
+            The output variables
+        heights: list of float, optional
+            The heights at which to search data
+        format_times_func: Function or 'default', optional
+            The function that maps state_coord values
+            to datetime dtype format
+        xr_read_pars: dict, optional
+            Parameters for xarray.open_dataset
+        kwargs: dict, optional
+            Parameters for the base class
+
+        """
+        super().__init__(
+            data_source,
+            *args,
+            heights=[],
+            pd_read_pars=None,
+            **kwargs,
+        )
+        self.state_coord = state_coord
+        self.heights = heights
+        self.h_coord = h_coord
+        self.xr_read_pars = xr_read_pars
+
+        if format_times_func == "default":
+            self.format_times_func = lambda t: t.astype("datetime64[ns]")
+        else:
+            self.format_times_func = format_times_func
+
+    def load_data(self, algo, verbosity=0):
+        """
+        Load and/or create all model data that is subject to chunking.
+
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        Returns
+        -------
+        idata: dict
+            The dict has exactly two entries: `data_vars`,
+            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
+            and `coords`, a dict with entries `dim_name_str -> dim_array`
+
+        """
+        if not isinstance(self.data_source, Dataset):
+            if not Path(self.data_source).is_file():
+                if verbosity:
+                    print(
+                        f"States '{self.name}': Reading static data '{self.data_source}' from context '{STATES}'"
+                    )
+                self.data_source = algo.dbook.get_file_path(
+                    STATES, self.data_source, check_raw=False
+                )
+                if verbosity:
+                    print(f"Path: {self.data_source}")
+            elif verbosity:
+                print(f"States '{self.name}': Reading file {self.data_source}")
+            data = open_dataset(self.data_source, **self.xr_read_pars)
+        else:
+            data = self.data_source
+
+        if self.states_sel is not None:
+            data = data.isel({self.state_coord: self.states_sel})
+        if self.states_loc is not None:
+            data = data.sel({self.state_coord: self.states_loc})
+
+        self._N = data.sizes[self.state_coord]
+        self._inds = data.coords[self.state_coord].to_numpy()
+        if self.format_times_func is not None:
+            self._inds = self.format_times_func(self._inds)
+
+        w_name = self.var2col.get(FV.WEIGHT, FV.WEIGHT)
+        self._weights = np.zeros((self._N, algo.n_turbines), dtype=FC.DTYPE)
+        if w_name in data.data_vars:
+            if data[w_name].dims != (self.state_coord,):
+                raise ValueError(
+                    f"Weights data '{w_name}': Expecting dims ({self.state_coord},), got {data[w_name]}"
+                )
+            self._weights[:] = data.data_vars[w_name].to_numpy()[:, None]
+        elif FV.WEIGHT in self.var2col:
+            raise KeyError(
+                f"Weight variable '{w_name}' defined in var2col, but not found in data_vars {list(data.data_vars.keys())}"
+            )
+        else:
+            self._weights = np.zeros((self._N, algo.n_turbines), dtype=FC.DTYPE)
+            self._weights[:] = 1.0 / self._N
+
+        cols = {}
+        self._solo = {}
+        for v in self.ovars:
+            c = self.var2col.get(v, v)
+            if c in self.fixed_vars:
+                pass
+            elif c in data.attrs:
+                self._solo[v] = np.full(self._N, data.attrs)
+            elif c in data.data_vars:
+                if data[c].dims == (self.state_coord,):
+                    self._solo[v] = data.data_vars[c].to_numpy()
+                elif data[c].dims == (self.state_coord, self.h_coord):
+                    cols[v] = c
+                else:
+                    raise ValueError(
+                        f"Variable '{c}': Expecting dims {(self.state_coord, self.h_coord)}, got {data[c].dims}"
+                    )
+            else:
+                raise KeyError(
+                    f"Missing variable '{c}', found data_vars {sorted(list(data.data_vars.keys()))} and attrs {sorted(list(data.attrs.keys()))}"
+                )
+
+        if self.heights is not None:
+            data = data.sel({self.h_coord: self.heights})
+        else:
+            self.heights = data[self.h_coord].to_numpy()
+
+        self.H = self.var(FV.H)
+        self.VARS = self.var("vars")
+        self.DATA = self.var("data")
+
+        idata = States.load_data(self, algo, verbosity)
+        idata["coords"][self.H] = self.heights
+        idata["coords"][self.VARS] = list(cols.keys())
+
+        dims = (FC.STATE, self.VARS, self.H)
+        idata["data_vars"][self.DATA] = (
+            dims,
+            np.stack(
+                [data.data_vars[c].to_numpy() for c in cols.values()], axis=1
+            ).astype(FC.DTYPE),
+        )
+
+        for v, d in self._solo.items():
+            idata["data_vars"][self.var(v)] = ((FC.STATE,), d.astype(FC.DTYPE))
+        self._solo = list(self._solo.keys())
+
+        return idata
 
 
 class MultiHeightTimeseries(MultiHeightStates):
     """
     Multi-height timeseries states data.
+
+    :group: input.states
+
     """
 
     RDICT = {"index_col": 0, "parse_dates": [0]}
+
+
+class MultiHeightNCTimeseries(MultiHeightNCStates):
+    """
+    Multi-height timeseries from xarray Dataset.
+
+    :group: input.states
+
+    """
+
+    def __init__(
+        self,
+        *args,
+        time_coord=FC.TIME,
+        **kwargs,
+    ):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        args: tuple, optional
+            Parameters for the base class
+        time_coord: str
+            Name of the state coordinate
+        kwargs: dict, optional
+            Parameters for the base class
+
+        """
+        super().__init__(*args, state_coord=time_coord, **kwargs)
