@@ -11,27 +11,33 @@ class ScanWS(States):
 
     Parameters
     ----------
-    ws_list : array_like
-        The wind speed values
-    wd : float
+    wd: float
         The wind direction
-    ti : float, optional
+    ti: float
         The TI value
-    rho : float, optional
+    rho: float
         The air density
 
-    Parameters
-    ----------
-    wd : float
-        The wind direction
-    ti : float
-        The TI value
-    rho : float
-        The air density
+    :group: input.states
 
     """
 
     def __init__(self, ws_list, wd, ti=None, rho=None):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        ws_list: array_like
+            The wind speed values
+        wd: float
+            The wind direction
+        ti: float, optional
+            The TI value
+        rho: float, optional
+            The air density
+
+        """
         super().__init__()
 
         self._wsl = np.array(ws_list)
@@ -40,26 +46,24 @@ class ScanWS(States):
         self.ti = ti
         self.rho = rho
 
-    def initialize(self, algo, verbosity=0):
+    def load_data(self, algo, verbosity=0):
         """
-        Initializes the model.
+        Load and/or create all model data that is subject to chunking.
 
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         Returns
         -------
-        idata : dict
+        idata: dict
             The dict has exactly two entries: `data_vars`,
             a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
             and `coords`, a dict with entries `dim_name_str -> dim_array`
@@ -67,9 +71,8 @@ class ScanWS(States):
         """
         self.WS = self.var(FV.WS)
 
-        idata = super().initialize(algo, verbosity)
-        self._update_idata(algo, idata)
-        idata["data_vars"][self.WS] = ((FV.STATE,), self._wsl)
+        idata = super().load_data(algo, verbosity)
+        idata["data_vars"][self.WS] = ((FC.STATE,), self._wsl)
 
         return idata
 
@@ -91,12 +94,12 @@ class ScanWS(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -115,18 +118,18 @@ class ScanWS(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights, shape: (n_states, n_turbines)
 
         """
         return np.full((self.N, algo.n_turbines), 1.0 / self.N, dtype=FC.DTYPE)
 
-    def calculate(self, algo, mdata, fdata, pdata):
+    def calculate(self, algo, mdata, fdata, tdata):
         """ "
         The main model calculation.
 
@@ -135,35 +138,31 @@ class ScanWS(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pdata : foxes.core.Data
-            The point data
+        tdata: foxes.core.TData
+            The target point data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
-            Values: numpy.ndarray with shape (n_states, n_points)
+            Values: numpy.ndarray with shape
+            (n_states, n_targets, n_tpoints)
 
         """
-        pdata[FV.WS][:] = mdata[self.WS][:, None]
+        tdata[FV.WS] = np.zeros_like(tdata[FC.TARGETS][..., 0])
+        tdata[FV.WS][:] = mdata[self.WS][:, None, None]
 
         if self.wd is not None:
-            pdata[FV.WD] = np.full(
-                (pdata.n_states, pdata.n_points), self.wd, dtype=FC.DTYPE
-            )
+            tdata[FV.WD] = np.full_like(tdata[FV.WS], self.wd)
         if self.ti is not None:
-            pdata[FV.TI] = np.full(
-                (pdata.n_states, pdata.n_points), self.ti, dtype=FC.DTYPE
-            )
+            tdata[FV.TI] = np.full_like(tdata[FV.WS], self.ti)
         if self.rho is not None:
-            pdata[FV.RHO] = np.full(
-                (pdata.n_states, pdata.n_points), self.rho, dtype=FC.DTYPE
-            )
+            tdata[FV.RHO] = np.full_like(tdata[FV.WS], self.rho)
 
-        return {v: pdata[v] for v in self.output_point_vars(algo)}
+        return {v: tdata[v] for v in self.output_point_vars(algo)}
