@@ -1,9 +1,10 @@
 import numpy as np
 import pandas as pd
+from xarray import Dataset
 from pathlib import Path
 
 from foxes.core import States, VerticalProfile
-from foxes.utils import PandasFileHelper
+from foxes.utils import PandasFileHelper, read_tab_file
 from foxes.data import STATES
 import foxes.variables as FV
 import foxes.constants as FC
@@ -13,50 +14,31 @@ class StatesTable(States):
     """
     States from a `pandas.DataFrame` or a pandas readable file.
 
-    Parameters
-    ----------
-    data_source : str or pandas.DataFrame
-        Either path to a file or data
-    output_vars : list of str
-        The output variables
-    var2col : dict
-        Mapping from variable names to data column names
-    fixed_vars : dict
-        Fixed uniform variable values, instead of
-        reading from data
-    profiles : dict
-        Key: output variable name str, Value: str or dict
-        or `foxes.core.VerticalProfile`
-    pd_read_pars : dict
-        pandas file reading parameters
-    states_sel : slice or range or list of int, optional
-        States subset selection
-    states_loc : list, optional
-        State index selection via pandas loc function
-
     Attributes
     ----------
-    data_source : str or pandas.DataFrame
+    data_source: str or pandas.DataFrame
         Either path to a file or data
-    ovars : list of str
+    ovars: list of str
         The output variables
-    var2col : dict
+    var2col: dict
         Mapping from variable names to data column names
-    fixed_vars : dict
+    fixed_vars: dict
         Fixed uniform variable values, instead of
         reading from data
-    profdicts : dict
+    profdicts: dict
         Key: output variable name str, Value: str or dict
         or `foxes.core.VerticalProfile`
-    rpars : dict
+    rpars: dict
         pandas file reading parameters
-    states_sel : slice or range or list of int
+    states_sel: slice or range or list of int
         States subset selection
-    states_loc : list
+    states_loc: list
         State index selection via pandas loc function
-    RDICT : dict
+    RDICT: dict
         Default pandas file reading parameters
 
+    :group: input.states
+
     """
 
     RDICT = {"index_col": 0}
@@ -72,6 +54,31 @@ class StatesTable(States):
         states_sel=None,
         states_loc=None,
     ):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        data_source: str or pandas.DataFrame
+            Either path to a file or data
+        output_vars: list of str
+            The output variables
+        var2col: dict
+            Mapping from variable names to data column names
+        fixed_vars: dict
+            Fixed uniform variable values, instead of
+            reading from data
+        profiles: dict
+            Key: output variable name str, Value: str or dict
+            or `foxes.core.VerticalProfile`
+        pd_read_pars: dict
+            pandas file reading parameters
+        states_sel: slice or range or list of int, optional
+            States subset selection
+        states_loc: list, optional
+            State index selection via pandas loc function
+
+        """
         super().__init__()
 
         self.data_source = data_source
@@ -84,11 +91,14 @@ class StatesTable(States):
         self.states_loc = states_loc
 
         if self.states_loc is not None and self.states_sel is not None:
-            raise ValueError(f"States '{self.name}': Cannot handle both 'states_sel' and 'states_loc', please pick one")
+            raise ValueError(
+                f"States '{self.name}': Cannot handle both 'states_sel' and 'states_loc', please pick one"
+            )
 
         self._weights = None
         self._N = None
         self._tvars = None
+        self._profiles = None
 
     def reset(self, algo=None, states_sel=None, states_loc=None, verbosity=0):
         """
@@ -96,13 +106,13 @@ class StatesTable(States):
 
         Parameters
         ----------
-        states_sel : slice or range or list of int, optional
+        states_sel: slice or range or list of int, optional
             States subset selection
-        states_loc : list, optional
+        states_loc: list, optional
             State index selection via pandas loc function
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
-            
+
         """
         if self.initialized:
             if algo is None:
@@ -112,27 +122,69 @@ class StatesTable(States):
             self.finalize(algo, verbosity)
         self.states_sel = states_sel
         self.states_loc = states_loc
-        
+
     def initialize(self, algo, verbosity=0):
         """
         Initializes the model.
 
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        self._profiles = {}
+        self._tvars = set(self.ovars)
+        for v, d in self.profdicts.items():
+            if isinstance(d, str):
+                self._profiles[v] = VerticalProfile.new(d)
+            elif isinstance(d, VerticalProfile):
+                self._profiles[v] = d
+            elif isinstance(d, dict):
+                t = d.pop("type")
+                self._profiles[v] = VerticalProfile.new(t, **d)
+            else:
+                raise TypeError(
+                    f"States '{self.name}': Wrong profile type '{type(d).__name__}' for variable '{v}'. Expecting VerticalProfile, str or dict"
+                )
+            self._tvars.update(self._profiles[v].input_vars())
+        self._tvars -= set(self.fixed_vars.keys())
+        self._tvars = list(self._tvars)
+        super().initialize(algo, verbosity)
+
+    def sub_models(self):
+        """
+        List of all sub-models
+
+        Returns
+        -------
+        smdls: list of str
+            Names of all sub models
+
+        """
+
+        return list(self._profiles.values())
+
+    def load_data(self, algo, verbosity=0):
+        """
+        Load and/or create all model data that is subject to chunking.
+
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         Returns
         -------
-        idata : dict
+        idata: dict
             The dict has exactly two entries: `data_vars`,
             a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
             and `coords`, a dict with entries `dim_name_str -> dim_array`
@@ -161,24 +213,6 @@ class StatesTable(States):
             data = PandasFileHelper().read_file(self.data_source, **rpars)
             isorg = False
 
-        self._profiles = {}
-        self._tvars = set(self.ovars)
-        for v, d in self.profdicts.items():
-            if isinstance(d, str):
-                self._profiles[v] = VerticalProfile.new(d)
-            elif isinstance(d, VerticalProfile):
-                self._profiles[v] = d
-            elif isinstance(d, dict):
-                t = d.pop("type")
-                self._profiles[v] = VerticalProfile.new(t, **d)
-            else:
-                raise TypeError(
-                    f"States '{self.name}': Wrong profile type '{type(d).__name__}' for variable '{v}'. Expecting VerticalProfile, str or dict"
-                )
-            self._tvars.update(self._profiles[v].input_vars())
-        self._tvars -= set(self.fixed_vars.keys())
-        self._tvars = list(self._tvars)
-
         if self.states_sel is not None:
             data = data.iloc[self.states_sel]
         elif self.states_loc is not None:
@@ -211,14 +245,10 @@ class StatesTable(States):
                 )
         data = data[tcols]
 
-        idata = super().initialize(algo, verbosity)
-        self._update_idata(algo, idata)
+        idata = super().load_data(algo, verbosity)
         idata["coords"][self.VARS] = self._tvars
-        idata["data_vars"][self.DATA] = ((FV.STATE, self.VARS), data.to_numpy())
+        idata["data_vars"][self.DATA] = ((FC.STATE, self.VARS), data.to_numpy())
 
-        algo.update_idata(list(self._profiles.values()),
-            idata=idata, verbosity=verbosity)
-            
         return idata
 
     def size(self):
@@ -239,24 +269,24 @@ class StatesTable(States):
 
         Returns
         -------
-        indices : array_like
+        indices: array_like
             The index labels of states, or None for default integers
 
         """
         return self._inds
-    
+
     def output_point_vars(self, algo):
         """
         The variables which are being modified by the model.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -268,18 +298,18 @@ class StatesTable(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights, shape: (n_states, n_turbines)
 
         """
         return self._weights
 
-    def calculate(self, algo, mdata, fdata, pdata):
+    def calculate(self, algo, mdata, fdata, tdata):
         """ "
         The main model calculation.
 
@@ -288,35 +318,43 @@ class StatesTable(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pdata : foxes.core.Data
-            The point data
+        tdata: foxes.core.TData
+            The target point data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
-            Values: numpy.ndarray with shape (n_states, n_points)
+            Values: numpy.ndarray with shape
+            (n_states, n_targets, n_tpoints)
 
         """
-        z = pdata[FV.POINTS][:, :, 2]
-
         for i, v in enumerate(self._tvars):
-            pdata[v][:] = mdata[self.DATA][:, i, None]
+            if v in tdata:
+                tdata[v][:] = mdata[self.DATA][:, i, None, None]
+            else:
+                tdata[v] = np.zeros(
+                    (tdata.n_states, tdata.n_targets, tdata.n_tpoints), dtype=FC.DTYPE
+                )
+                tdata[v][:] = mdata[self.DATA][:, i, None, None]
+                tdata.dims[v] = (FC.STATE, FC.TARGET, FC.TPOINT)
 
         for v, f in self.fixed_vars.items():
-            pdata[v] = np.full((pdata.n_states, pdata.n_points), f, dtype=FC.DTYPE)
+            tdata[v] = np.full(
+                (tdata.n_states, tdata.n_targets, tdata.n_tpoints), f, dtype=FC.DTYPE
+            )
 
+        z = tdata[FC.TARGETS][..., 2]
         for v, p in self._profiles.items():
-            pres = p.calculate(pdata, z)
-            pdata[v] = pres
+            tdata[v] = p.calculate(tdata, z)
 
-        return {v: pdata[v] for v in self.output_point_vars(algo)}
+        return {v: tdata[v] for v in self.output_point_vars(algo)}
 
     def finalize(self, algo, verbosity=0):
         """
@@ -324,9 +362,9 @@ class StatesTable(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level
 
         """
@@ -340,6 +378,124 @@ class StatesTable(States):
 class Timeseries(StatesTable):
     """
     Timeseries states data.
+
+    :group: input.states
+
     """
 
     RDICT = {"index_col": 0, "parse_dates": [0]}
+
+
+class TabStates(StatesTable):
+    """
+    States created from a single tab file
+
+    :group: input.states
+
+    """
+
+    def __init__(self, data_source, *args, normalize=True, **kwargs):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        data_source: str or xarray.Dataset
+            The tab file data file name, or its data
+        args: tuple, optional
+            Additional parameters for StatesTable
+        normalize: bool
+            Normalize the tab file data
+        kwargs: dict, optional
+            Additional parameters for StatesTable
+
+        """
+        self._normalize = normalize
+        if isinstance(data_source, Dataset):
+            self._tab_source = None
+            self._tab_data = data_source
+        elif isinstance(data_source, (str, Path)):
+            self._tab_source = data_source
+            self._tab_data = None
+        else:
+            raise TypeError(
+                f"Expecting str, Path or xarray.Dataset as data_source, got {type(data_source)}"
+            )
+
+        super().__init__(data_source=None, *args, **kwargs)
+
+    def load_data(self, algo, verbosity=0):
+        """
+        Load and/or create all model data that is subject to chunking.
+
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        Returns
+        -------
+        idata: dict
+            The dict has exactly two entries: `data_vars`,
+            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
+            and `coords`, a dict with entries `dim_name_str -> dim_array`
+
+        """
+        if self.data_source is None:
+            if self._tab_data is None:
+                if not Path(self._tab_source).is_file():
+                    if verbosity:
+                        print(
+                            f"States '{self.name}': Reading static data '{self._tab_source}' from context '{STATES}'"
+                        )
+                    self._tab_source = algo.dbook.get_file_path(
+                        STATES, self._tab_source, check_raw=False
+                    )
+                    if verbosity:
+                        print(f"Path: {self._tab_source}")
+                elif verbosity:
+                    print(f"States '{self.name}': Reading file {self._tab_source}")
+                self._tab_data = read_tab_file(self._tab_source, self._normalize)
+
+            a = self._tab_data.attrs["factor_ws"]
+            b = self._tab_data.attrs["shift_wd"]
+            if b != 0.0:
+                raise ValueError(
+                    f"{self.name}: shift_wd = {b} is not supported, expecting zero"
+                )
+
+            wd0 = self._tab_data["wd"].to_numpy()
+            ws0 = a * np.append(
+                np.array([0], dtype=FC.DTYPE), self._tab_data["ws"].to_numpy()
+            )
+            ws0 = 0.5 * (ws0[:-1] + ws0[1:])
+
+            n_ws = self._tab_data.sizes["ws"]
+            n_wd = self._tab_data.sizes["wd"]
+            ws = np.zeros((n_ws, n_wd), dtype=FC.DTYPE)
+            wd = np.zeros((n_ws, n_wd), dtype=FC.DTYPE)
+            ws[:] = ws0[:, None]
+            wd[:] = wd0[None, :]
+
+            wd_freq = self._tab_data["wd_freq"].to_numpy() / 100
+            weights = self._tab_data["ws_freq"].to_numpy() * wd_freq[None, :] / 1000
+
+            sel = weights > 0
+
+            self.data_source = pd.DataFrame(
+                index=np.arange(np.sum(sel)),
+                data={
+                    FV.WS: ws[sel],
+                    FV.WD: wd[sel],
+                    FV.WEIGHT: weights[sel],
+                },
+            )
+            self.data_source.index.name = FC.STATE
+
+        return super().load_data(algo, verbosity)
