@@ -9,54 +9,43 @@ class Turbine2FarmModel(FarmModel):
     into farm models, simply by selecting
     all turbines.
 
-    Parameters
-    ----------
-    turbine_model : foxes.core.TurbineModel
-        The turbine model
-
     Attributes
     ----------
-    turbine_model : foxes.core.TurbineModel
+    turbine_model: foxes.core.TurbineModel
         The turbine model
 
+    :group: models.farm_models
+
     """
 
     def __init__(self, turbine_model):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        turbine_model: foxes.core.TurbineModel
+            The turbine model
+
+        """
         super().__init__()
         self.turbine_model = turbine_model
+        self.name = turbine_model.name + "_t2f"
 
     def __repr__(self):
         return f"{type(self).__name__}({self.turbine_model})"
 
-    def initialize(self, algo, verbosity=0):
+    def sub_models(self):
         """
-        Initializes the model.
-
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level, 0 = silent
+        List of all sub-models
 
         Returns
         -------
-        idata : dict
-            The dict has exactly two entries: `data_vars`,
-            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
-            and `coords`, a dict with entries `dim_name_str -> dim_array`
+        smdls: list of foxes.core.Model
+            Names of all sub models
 
         """
-        idata = super().initialize(algo, verbosity)
-        algo.update_idata(self.turbine_model, idata=idata, verbosity=verbosity)
-
-        return idata
+        return [self.turbine_model]
 
     def output_farm_vars(self, algo):
         """
@@ -64,12 +53,12 @@ class Turbine2FarmModel(FarmModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -84,37 +73,21 @@ class Turbine2FarmModel(FarmModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        **parameters : dict, optional
+        **parameters: dict, optional
             Init parameters forwarded to the turbine model
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray with shape (n_states, n_turbines)
 
         """
         s = np.ones((algo.n_states, algo.n_turbines), dtype=bool)
         return self.turbine_model.calculate(algo, mdata, fdata, st_sel=s, **parameters)
-
-    def finalize(self, algo, verbosity=0):
-        """
-        Finalizes the model.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        verbosity : int
-            The verbosity level
-
-        """
-        if self.turbine_model.initialized:
-            algo.finalize_model(self.turbine_model, verbosity)
-        super().finalize(algo, verbosity)
