@@ -1,6 +1,10 @@
+"""
+Partial wake models.
+"""
+
+from .centre import PartialCentre
 from .rotor_points import RotorPoints
 from .top_hat import PartialTopHat
 from .axiwake import PartialAxiwake
-from .distsliced import PartialDistSlicedWake
+from .segregated import PartialSegregated
 from .grid import PartialGrid
-from .mapped import Mapped
