@@ -10,30 +10,19 @@ class SetUniformData(PointDataModel):
     """
     Set uniform data (can be state dependent)
 
-    Parameters
-    ----------
-    data_source : str or pandas.DataFrame or dict
-        Either a file name, or a data frame, both assuming
-        state dependent data. Or a dict for state independent
-        uniform data (i.e., scalars)
-    output_vars : list of str
-        The variables to be written
-    var2col : dict
-        Mapping from variable names to data column names
-    pd_read_pars : dict
-        pandas file reading parameters
-
     Attributes
     ----------
-    data_source : str or pandas.DataFrame or dict
+    data_source: str or pandas.DataFrame or dict
         Either a file name, or a data frame, both assuming
         state dependent data. Or a dict for state independent
         uniform data (i.e., scalars)
-    ovars : list of str
+    ovars: list of str
         The variables to be written
-    var2col : dict
+    var2col: dict
         Mapping from variable names to data column names
 
+    :group: models.point_models
+
     """
 
     def __init__(
@@ -43,32 +32,47 @@ class SetUniformData(PointDataModel):
         var2col={},
         pd_read_pars={},
     ):
-        self.data_source = data_source
-        self.ovars = output_vars
-        self.var2col = var2col
-        
-        self._rpars = pd_read_pars
-
-    def initialize(self, algo, verbosity=0):
         """
-        Initializes the model.
-
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
+        Constructor.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        data_source: str or pandas.DataFrame or dict
+            Either a file name, or a data frame, both assuming
+            state dependent data. Or a dict for state independent
+            uniform data (i.e., scalars)
+        output_vars: list of str
+            The variables to be written
+        var2col: dict
+            Mapping from variable names to data column names
+        pd_read_pars: dict
+            pandas file reading parameters
+
+        """
+        self.data_source = data_source
+        self.ovars = output_vars
+        self.var2col = var2col
+
+        self._rpars = pd_read_pars
+
+    def load_data(self, algo, verbosity=0):
+        """
+        Load and/or create all model data that is subject to chunking.
+
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         Returns
         -------
-        idata : dict
+        idata: dict
             The dict has exactly two entries: `data_vars`,
             a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
             and `coords`, a dict with entries `dim_name_str -> dim_array`
@@ -89,13 +93,11 @@ class SetUniformData(PointDataModel):
             rpars = dict(index_col=0)
             rpars.update(self._rpars)
             data = PandasFileHelper().read_file(self.data_source, **rpars)
-            data = data[
-                [self.var2col.get(v, v) for v in self.ovars]
-            ].to_numpy(FC.DTYPE)
+            data = data[[self.var2col.get(v, v) for v in self.ovars]].to_numpy(FC.DTYPE)
 
-        idata = super().initialize(algo, verbosity)
+        idata = super().load_data(algo, verbosity)
         idata["coords"][self.VARS] = self.ovars
-        idata["data_vars"][self.DATA] = ((FV.STATE, self.VARS), data)
+        idata["data_vars"][self.DATA] = ((FC.STATE, self.VARS), data)
 
         return idata
 
@@ -105,12 +107,12 @@ class SetUniformData(PointDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -125,18 +127,18 @@ class SetUniformData(PointDataModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        pdata : foxes.core.Data
-            The point data
+        tdata: foxes.core.TData
+            The target point data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray with shape (n_states, n_points)
 
