@@ -6,26 +6,9 @@ import foxes.constants as FC
 
 class GridRotor(RotorModel):
     """
-    The regular grid rotor model, composed maximally
+    The weighted regular grid rotor model, composed maximally
     of n x n points, possibly kicking out the outside points.
 
-    Parameters
-    ----------
-    n: int
-        The number of points along one direction,
-        maximal number of points is N = n * n
-    calc_vars : list of str
-        The variables that are calculated by the model
-        (Their ambients are added automatically)
-    reduce: bool
-        Flag for reduction to points actually representing
-        an area with overlap with the circe, recalculating
-        the self.weights accordingly
-    nint: int
-        Integration steps per element
-    name: str, optional
-        The model name
-
     Attributes
     ----------
     n: int
@@ -38,9 +21,32 @@ class GridRotor(RotorModel):
     nint: int
         Integration steps per element
 
+    :group: models.rotor_models
+
     """
 
     def __init__(self, n, calc_vars, reduce=True, nint=200):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        n: int
+            The number of points along one direction,
+            maximal number of points is N = n * n
+        calc_vars: list of str
+            The variables that are calculated by the model
+            (Their ambients are added automatically)
+        reduce: bool
+            Flag for reduction to points actually representing
+            an area with overlap with the circe, recalculating
+            the self.weights accordingly
+        nint: int
+            Integration steps per element
+        name: str, optional
+            The model name
+
+        """
         super().__init__(calc_vars)
 
         self.n = n
@@ -48,33 +54,23 @@ class GridRotor(RotorModel):
         self.nint = nint
 
     def __repr__(self):
-        return super().__repr__() + f"(n={self.n})"
+        r = "" if self.reduce else ", reduce=False"
+        return f"{type(self).__name__}(n={self.n}){r}"
 
     def initialize(self, algo, verbosity=0):
         """
         Initializes the model.
 
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
-
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
-        Returns
-        -------
-        idata : dict
-            The dict has exactly two entries: `data_vars`,
-            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
-            and `coords`, a dict with entries `dim_name_str -> dim_array`
-
         """
+        super().initialize(algo, verbosity)
+
         N = self.n * self.n
         delta = 2.0 / self.n
         x = [-1.0 + (i + 0.5) * delta for i in range(self.n)]
@@ -85,11 +81,9 @@ class GridRotor(RotorModel):
         self.dpoints[:, 2] = y.reshape(N)
 
         if self.reduce:
-
             self.weights = np.zeros((self.n, self.n), dtype=FC.DTYPE)
             for i in range(0, self.n):
                 for j in range(0, self.n):
-
                     d = delta / self.nint
                     hx = [
                         x[i, j] - delta / 2.0 + (k + 0.5) * d for k in range(self.nint)
@@ -110,20 +104,17 @@ class GridRotor(RotorModel):
             self.weights /= np.sum(self.weights)
 
         else:
-
             self.dpoints[:, 1] = x.reshape(N)
             self.dpoints[:, 2] = y.reshape(N)
             self.weights = np.ones(N, dtype=FC.DTYPE) / N
 
-        return super().initialize(algo, verbosity)
-
     def n_rotor_points(self):
         """
         The number of rotor points
 
         Returns
         -------
-        n_rpoints : int
+        n_rpoints: int
             The number of rotor points
 
         """
@@ -142,7 +133,7 @@ class GridRotor(RotorModel):
 
         Returns
         -------
-        dpoints : numpy.ndarray
+        dpoints: numpy.ndarray
             The design points, shape: (n_points, 3)
 
         """
@@ -154,7 +145,7 @@ class GridRotor(RotorModel):
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights of the rotor points,
             add to one, shape: (n_rpoints,)
 
