@@ -1,4 +1,7 @@
-from .set_XYHD import SetXYHD
+"""
+Turbine models.
+"""
+
 from .kTI_model import kTI
 from .set_farm_vars import SetFarmVars
 from .thrust2ct import Thrust2Ct
@@ -8,3 +11,5 @@ from .table_factors import TableFactors
 from .yaw2yawm import YAW2YAWM
 from .yawm2yaw import YAWM2YAW
 from .calculator import Calculator
+from .rotor_centre_calc import RotorCentreCalc
+from .lookup_table import LookupTable
