@@ -18,27 +18,50 @@ class PowerMask(TurbineModel):
     so it is wise to use it after calling the
     turbine type model.
 
-    Parameters
-    ----------
-    var_ws_P : str
-        The wind speed variable for power lookup
-    factor_P : float
-        The power unit factor, e.g. 1000 for kW
-
     Attributes
     ----------
-    var_ws_P : str
+    var_ws_P: str
         The wind speed variable for power lookup
-    factor_P : float
+    factor_P: float
         The power unit factor, e.g. 1000 for kW
+    P_lim: float
+        Threshold power delta for boosts
+    induction: foxes.core.AxialInductionModel
+        The induction model
+
+    :group: models.turbine_models
 
     """
 
-    def __init__(self, var_ws_P=FV.REWS3, factor_P=1.0e3):
+    def __init__(self, var_ws_P=FV.REWS3, factor_P=1.0e3, P_lim=100, induction="Betz"):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        var_ws_P: str
+            The wind speed variable for power lookup
+        factor_P: float
+            The power unit factor, e.g. 1000 for kW
+        P_lim: float
+            Threshold power delta for boosts
+        induction: foxes.core.AxialInductionModel or str
+            The induction model
+
+        """
         super().__init__()
 
         self.var_ws_P = var_ws_P
         self.factor_P = factor_P
+        self.P_lim = P_lim
+        self.induction = induction
+
+    def __repr__(self):
+        iname = (
+            self.induction if isinstance(self.induction, str) else self.induction.name
+        )
+        a = f"var_ws_P={self.var_ws_P}, P_lim={self.P_lim}, induction={iname}"
+        return f"{type(self).__name__}({a})"
 
     def output_farm_vars(self, algo):
         """
@@ -46,83 +69,117 @@ class PowerMask(TurbineModel):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
         return [FV.P, FV.CT]
 
+    def sub_models(self):
+        """
+        List of all sub-models
+
+        Returns
+        -------
+        smdls: list of foxes.core.Model
+            All sub models
+
+        """
+        return [self.induction]
+
     def initialize(self, algo, verbosity=0):
         """
         Initializes the model.
 
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
-
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
-        Returns
-        -------
-        idata : dict
-            The dict has exactly two entries: `data_vars`,
-            a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
-            and `coords`, a dict with entries `dim_name_str -> dim_array`
-
         """
+        if isinstance(self.induction, str):
+            self.induction = algo.mbook.axial_induction[self.induction]
+        super().initialize(algo, verbosity)
+
         self._P_rated = []
         for t in algo.farm_controller.turbine_types:
             Pnom = FC.DTYPE(t.P_nominal)
             if np.isnan(Pnom):
-                raise ValueError(f"Model '{self.name}': P_nominal is NaN for turbine type '{t.name}'")
+                raise ValueError(
+                    f"Model '{self.name}': P_nominal is NaN for turbine type '{t.name}'"
+                )
             self._P_rated.append(Pnom)
         self._P_rated = np.array(self._P_rated, dtype=FC.DTYPE)
-        
-        return super().initialize(algo, verbosity)
 
-    @classmethod
-    def update_P_ct(cls, data, max_P, rated_P, factor_P, var_ws=FV.REWS3, P_lim=100):
+    def calculate(self, algo, mdata, fdata, st_sel):
+        """
+        The main model calculation.
+
+        This function is executed on a single chunk of data,
+        all computations should be based on numpy arrays.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        mdata: foxes.core.MData
+            The model data
+        fdata: foxes.core.FData
+            The farm data
+        st_sel: slice or numpy.ndarray of bool
+            The state-turbine selection,
+            for shape: (n_states, n_turbines)
+
+        Returns
+        -------
+        results: dict
+            The resulting data, keys: output variable str.
+            Values: numpy.ndarray with shape (n_states, n_turbines)
+
+        """
+        # prepare:
+        P = fdata[FV.P]
+        max_P = fdata[FV.MAX_P]
+        P_rated = self._P_rated[None, :]
 
         # select power entries for which this is active:
-        P = data[FV.P]
-        sel = ~np.isnan(max_P) & (
-            ((max_P < rated_P) & (P > max_P))
-            | ((max_P > rated_P) & (P > rated_P - P_lim))
+        sel = np.zeros((fdata.n_states, fdata.n_turbines), dtype=bool)
+        sel[st_sel] = True
+        sel = (
+            sel
+            & ~np.isnan(max_P)
+            & (
+                ((max_P < P_rated) & (P > max_P))
+                | ((max_P > P_rated) & (P > P_rated - self.P_lim))
+            )
         )
         if np.any(sel):
-
             # apply selection:
             max_P = max_P[sel]
-            ws = data[var_ws][sel]
-            rho = data[FV.RHO][sel]
-            r = data[FV.D][sel] / 2
+            ws = fdata[self.var_ws_P][sel]
+            rho = fdata[FV.RHO][sel]
+            r = fdata[FV.D][sel] / 2
             P = P[sel]
-            ct = data[FV.CT][sel]
-            ct[ct > 1.0] = 1.0
+            ct = fdata[FV.CT][sel]
 
             # calculate power efficiency e of turbine
             # e is the ratio of the cp derived from the power curve
             # and the theoretical cp from the turbine induction
-            cp = P / (0.5 * ws**3 * rho * np.pi * r**2) * factor_P
-            a = 0.5 * (1 - np.sqrt(1 - ct))
+            cp = P / (0.5 * ws**3 * rho * np.pi * r**2) * self.factor_P
+            a = self.induction.ct2a(ct)
             cp_a = 4 * a**3 - 8 * a**2 + 4 * a
             e = cp / cp_a
             del cp, a, cp_a, ct, P
 
             # calculating new cp for changed power
-            cp = max_P / (0.5 * ws**3 * rho * np.pi * r**2) * factor_P
+            cp = max_P / (0.5 * ws**3 * rho * np.pi * r**2) * self.factor_P
 
             # find roots:
             N = len(cp)
@@ -137,43 +194,9 @@ class PowerMask(TurbineModel):
             del a0, a1, a2, a3, rts
 
             # set results:
-            P = data[FV.P]
-            ct = data[FV.CT]
+            P = fdata[FV.P]
+            ct = fdata[FV.CT]
             P[sel] = max_P
             ct[sel] = 4 * a * (1 - a)
 
-    def calculate(self, algo, mdata, fdata, st_sel):
-        """ "
-        The main model calculation.
-
-        This function is executed on a single chunk of data,
-        all computations should be based on numpy arrays.
-
-        Parameters
-        ----------
-        algo : foxes.core.Algorithm
-            The calculation algorithm
-        mdata : foxes.core.Data
-            The model data
-        fdata : foxes.core.Data
-            The farm data
-        st_sel : numpy.ndarray of bool
-            The state-turbine selection,
-            shape: (n_states, n_turbines)
-
-        Returns
-        -------
-        results : dict
-            The resulting data, keys: output variable str.
-            Values: numpy.ndarray with shape (n_states, n_turbines)
-
-        """
-        
-        # prepare:
-        max_P = fdata[FV.MAX_P]
-        rated_P = self._P_rated[None, :]
-
-        # calculate:
-        self.update_P_ct(fdata, max_P, rated_P, self.factor_P, var_ws=self.var_ws_P)
-
         return {FV.P: fdata[FV.P], FV.CT: fdata[FV.CT]}
