@@ -11,19 +11,25 @@ class RotorWD(WakeFrame):
     Align the first axis for each rotor with the
     local normalized wind direction.
 
-    Parameters
-    ----------
-    var_wd : str
-        The wind direction variable
-
     Attributes
     ----------
-    var_wd : str
+    var_wd: str
         The wind direction variable
 
+    :group: models.wake_frames
+
     """
 
     def __init__(self, var_wd=FV.WD):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        var_wd: str
+            The wind direction variable
+
+        """
         super().__init__()
         self.var_wd = var_wd
 
@@ -36,58 +42,65 @@ class RotorWD(WakeFrame):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
 
         Returns
         -------
-        order : numpy.ndarray
+        order: numpy.ndarray
             The turbine order, shape: (n_states, n_turbines)
 
         """
         n = np.mean(wd2uv(fdata[self.var_wd], axis=1), axis=-1)
         xy = fdata[FV.TXYH][:, :, :2]
-
         order = np.argsort(np.einsum("std,sd->st", xy, n), axis=-1)
 
         return order
 
-    def get_wake_coos(self, algo, mdata, fdata, states_source_turbine, points):
+    def get_wake_coos(
+        self,
+        algo,
+        mdata,
+        fdata,
+        tdata,
+        downwind_index,
+    ):
         """
-        Calculate wake coordinates.
+        Calculate wake coordinates of rotor points.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        states_source_turbine : numpy.ndarray
-            For each state, one turbine index for the
-            wake causing turbine. Shape: (n_states,)
-        points : numpy.ndarray
-            The evaluation points, shape: (n_states, n_points, 3)
+        tdata: foxes.core.TData
+            The target point data
+        downwind_index: int
+            The index of the wake causing turbine
+            in the downwnd order
 
         Returns
         -------
-        wake_coos : numpy.ndarray
-            The wake coordinates, shape: (n_states, n_points, 3)
+        wake_coos: numpy.ndarray
+            The wake frame coordinates of the evaluation
+            points, shape: (n_states, n_targets, n_tpoints, 3)
 
         """
-        n_states = mdata.n_states
-        stsel = (np.arange(n_states), states_source_turbine)
+        n_states = tdata.n_states
+        targets = tdata[FC.TARGETS]
 
-        xyz = fdata[FV.TXYH][stsel]
-        delta = points - xyz[:, None, :]
+        xyz = fdata[FV.TXYH][:, downwind_index]
+        delta = targets - xyz[:, None, None, :]
         del xyz
 
-        wd = fdata[self.var_wd][stsel]
+        wd = fdata[self.var_wd][:, downwind_index]
 
         nax = np.zeros((n_states, 3, 3), dtype=FC.DTYPE)
         n = nax[:, 0, :2]
@@ -97,40 +110,36 @@ class RotorWD(WakeFrame):
         nax[:, 2, 2] = 1
         del wd
 
-        coos = np.einsum("spd,sad->spa", delta, nax)
+        coos = np.einsum("stpd,sad->stpa", delta, nax)
 
         return coos
 
-    def get_centreline_points(self, algo, mdata, fdata, states_source_turbine, x):
+    def get_centreline_points(self, algo, mdata, fdata, downwind_index, x):
         """
         Gets the points along the centreline for given
         values of x.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.MData
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.FData
             The farm data
-        states_source_turbine : numpy.ndarray
-            For each state, one turbine index for the
-            wake causing turbine. Shape: (n_states,)
-        x : numpy.ndarray
+        downwind_index: int
+            The index in the downwind order
+        x: numpy.ndarray
             The wake frame x coordinates, shape: (n_states, n_points)
-        
+
         Returns
         -------
-        points : numpy.ndarray
+        points: numpy.ndarray
             The centreline points, shape: (n_states, n_points, 3)
 
         """
-        n_states = mdata.n_states
-        stsel = (np.arange(n_states), states_source_turbine)
-
-        wd = fdata[self.var_wd][stsel]
+        wd = fdata[self.var_wd][:, downwind_index]
         n = np.append(wd2uv(wd, axis=-1), np.zeros_like(wd)[:, None], axis=-1)
 
-        xyz = fdata[FV.TXYH][stsel]
+        xyz = fdata[FV.TXYH][:, downwind_index]
         return xyz[:, None, :] + x[:, :, None] * n[:, None, :]
