@@ -1,3 +1,7 @@
+"""
+Wake models.
+"""
+
 from .dist_sliced import DistSlicedWakeModel
 from .axisymmetric import AxisymmetricWakeModel
 from .top_hat import TopHatWakeModel
@@ -5,3 +9,4 @@ from .gaussian import GaussianWakeModel
 
 from . import wind
 from . import ti
+from . import induction
