@@ -1,7 +1,8 @@
+import numpy as np
 from iwopy import Problem
 
 from foxes.utils.runners import DefaultRunner
-import foxes.variables as FV
+import foxes.constants as FC
 from .pop_states import PopStates
 
 
@@ -9,30 +10,18 @@ class FarmOptProblem(Problem):
     """
     Abstract base class of wind farm optimization problems.
 
-    Parameters
-    ----------
-    name : str
-        The problem's name
-    algo : foxes.core.Algorithm
-        The algorithm
-    runner : foxes.core.Runner, optional
-        The runner for running the algorithm
-    sel_turbines : list of int, optional
-        The turbines selected for optimization,
-        or None for all
-    calc_farm_args : dict
-        Additional parameters for algo.calc_farm()
-    kwargs : dict, optional
-        Additional parameters for `iwopy.Problem`
-
     Attributes
     ----------
-    algo : foxes.core.Algorithm
+    algo: foxes.core.Algorithm
         The algorithm
-    runner : foxes.core.Runner
+    runner: foxes.core.Runner
         The runner for running the algorithm
-    calc_farm_args : dict
+    calc_farm_args: dict
         Additional parameters for algo.calc_farm()
+    points : numpy.ndarray
+        The probe points, shape: (n_states, n_points, 3)
+
+    :group: opt.core
 
     """
 
@@ -43,13 +32,37 @@ class FarmOptProblem(Problem):
         runner=None,
         sel_turbines=None,
         calc_farm_args={},
+        points=None,
         **kwargs,
     ):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        name: str
+            The problem's name
+        algo: foxes.core.Algorithm
+            The algorithm
+        runner: foxes.core.Runner, optional
+            The runner for running the algorithm
+        sel_turbines: list of int, optional
+            The turbines selected for optimization,
+            or None for all
+        calc_farm_args: dict
+            Additional parameters for algo.calc_farm()
+        points : numpy.ndarray, optional
+            The probe points, shape: (n_states, n_points, 3)
+        kwargs: dict, optional
+            Additional parameters for `iwopy.Problem`
+
+        """
         super().__init__(name, **kwargs)
 
         self.algo = algo
         self.runner = runner
         self.calc_farm_args = calc_farm_args
+        self.points = points
 
         self._sel_turbines = sel_turbines
         self._count = None
@@ -78,7 +91,11 @@ class FarmOptProblem(Problem):
             Indices of the selected turbines
 
         """
-        return self._sel_turbines if self._sel_turbines is not None else list(range(self.farm.n_turbines))
+        return (
+            self._sel_turbines
+            if self._sel_turbines is not None
+            else list(range(self.farm.n_turbines))
+        )
 
     @property
     def n_sel_turbines(self):
@@ -105,7 +122,7 @@ class FarmOptProblem(Problem):
 
         """
         return len(self.sel_turbines) == self.algo.n_turbines
-    
+
     @property
     def counter(self):
         """
@@ -126,9 +143,9 @@ class FarmOptProblem(Problem):
 
         Parameters
         ----------
-        var : str
+        var: str
             The variable name
-        turbine_i : int
+        turbine_i: int
             The turbine index
 
         Returns
@@ -147,65 +164,27 @@ class FarmOptProblem(Problem):
 
         Parameters
         ----------
-        tvr : str
+        tvr: str
             The turbine variable name
 
         Returns
         -------
-        var : str
+        var: str
             The foxes variable name
-        turbine_i : int
+        turbine_i: int
             The turbine index
 
         """
         t = tvr.split("_")
         return t[0], int(t[1])
 
-    def _update_keep_models(self, drop_vars, exclude=None, verbosity=1):
-        """
-        Updates algo.keep_models during initialization
-
-        Parameters
-        ----------
-        drop_vars : list of str
-            Variables that decide about dropping model
-            from algo.keep_models
-        exclude : list of str, optional
-            The model names to be excluded, default
-            is original states and problem model names
-        verbosity : int
-            The verbosity level, 0 = silent
-
-        """
-        if exclude is None:
-            exclude = [self._org_states_name]
-        self.algo.keep_models.append(self._org_states_name)
-        for mname, idata in self.algo.idata_mem.items():
-            if mname not in exclude and self.name not in mname:
-                keep = True
-                for d in idata["data_vars"].values():
-                    for dropv in drop_vars:
-                        if dropv in d[0]:
-                            keep = False
-                            break
-                    if not keep:
-                        break
-                if keep:
-                    self.algo.keep_models.append(mname)
-
-    def initialize(self, drop_vars=[FV.STATE], exclude=None, verbosity=1):
+    def initialize(self, verbosity=1):
         """
         Initialize the object.
 
         Parameters
         ----------
-        drop_vars : list of str
-            Variables that decide about dropping model
-            from algo.keep_models
-        exclude : list of str, optional
-            The model names to be excluded, default
-            is original states and problem model names
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         """
@@ -220,9 +199,6 @@ class FarmOptProblem(Problem):
         self._org_states_name = self.algo.states.name
         self._org_n_states = self.algo.n_states
 
-        # keep models that do not contain state related data,
-        # and always the original states:
-        self._update_keep_models(drop_vars, exclude, verbosity)
         self.algo.finalize()
 
         self._count = 0
@@ -237,20 +213,20 @@ class FarmOptProblem(Problem):
             if self.algo.initialized:
                 self.algo.finalize()
             self.algo.states = states
-    
+
     def update_problem_individual(self, vars_int, vars_float):
         """
         Update the algo and other data using
         the latest optimization variables.
-        
-        This function is called before running the farm 
+
+        This function is called before running the farm
         calculation.
 
         Parameters
         ----------
-        vars_int : np.array
+        vars_int: np.array
             The integer variable values, shape: (n_vars_int,)
-        vars_float : np.array
+        vars_float: np.array
             The float variable values, shape: (n_vars_float,)
 
         """
@@ -263,15 +239,15 @@ class FarmOptProblem(Problem):
         """
         Update the algo and other data using
         the latest optimization variables.
-        
-        This function is called before running the farm 
+
+        This function is called before running the farm
         calculation.
 
         Parameters
         ----------
-        vars_int : np.array
+        vars_int: np.array
             The integer variable values, shape: (n_pop, n_vars_int,)
-        vars_float : np.array
+        vars_float: np.array
             The float variable values, shape: (n_pop, n_vars_float,)
 
         """
@@ -289,21 +265,29 @@ class FarmOptProblem(Problem):
 
         Parameters
         ----------
-        vars_int : np.array
+        vars_int: np.array
             The integer variable values, shape: (n_vars_int,)
-        vars_float : np.array
+        vars_float: np.array
             The float variable values, shape: (n_vars_float,)
 
         Returns
         -------
-        problem_results : Any
+        problem_results: Any
             The results of the variable application
             to the problem
 
-        """       
+        """
         self._count += 1
         self.update_problem_individual(vars_int, vars_float)
-        return self.runner.run(self.algo.calc_farm, kwargs=self.calc_farm_args)
+        farm_results = self.runner.run(self.algo.calc_farm, kwargs=self.calc_farm_args)
+
+        if self.points is None:
+            return farm_results
+        else:
+            point_results = self.runner.run(
+                self.algo.calc_points, args=(farm_results, self.points)
+            )
+            return farm_results, point_results
 
     def apply_population(self, vars_int, vars_float):
         """
@@ -312,26 +296,37 @@ class FarmOptProblem(Problem):
 
         Parameters
         ----------
-        vars_int : np.array
+        vars_int: np.array
             The integer variable values, shape: (n_pop, n_vars_int)
-        vars_float : np.array
+        vars_float: np.array
             The float variable values, shape: (n_pop, n_vars_float)
 
         Returns
         -------
-        problem_results : Any
+        problem_results: Any
             The results of the variable application
             to the problem
 
         """
         self._count += 1
-        
-        self.update_problem_population(vars_int, vars_float)
-        results = self.runner.run(self.algo.calc_farm, kwargs=self.calc_farm_args)
-        results["n_pop"] = len(vars_float)
-        results["n_org_states"] = self._org_n_states
 
-        return results
+        self.update_problem_population(vars_int, vars_float)
+        farm_results = self.runner.run(self.algo.calc_farm, kwargs=self.calc_farm_args)
+        farm_results["n_pop"] = len(vars_float)
+        farm_results["n_org_states"] = self._org_n_states
+
+        if self.points is None:
+            return farm_results
+        else:
+            n_pop = farm_results["n_pop"].values
+            n_states, n_points = self.points.shape[:2]
+            pop_points = np.zeros((n_pop, n_states, n_points, 3), dtype=FC.DTYPE)
+            pop_points[:] = self.points[None, :, :, :]
+            pop_points = pop_points.reshape(n_pop * n_states, n_points, 3)
+            point_results = self.runner.run(
+                self.algo.calc_points, args=(farm_results, pop_points)
+            )
+            return farm_results, point_results
 
     def add_to_layout_figure(self, ax, **kwargs):
         """
@@ -339,7 +334,7 @@ class FarmOptProblem(Problem):
 
         Parameters
         ----------
-        ax : matplotlib.pyplot.Axis
+        ax: matplotlib.pyplot.Axis
             The figure axis
 
         """
