@@ -11,78 +11,76 @@ class PopStates(States):
     calculation, via artificial states of length
     n_pop times n_states.
 
-    Parameters
-    ----------
-    states : foxes.core.States
-        The original states
-    n_pop : int
-        The population size
-
     Attributes
     ----------
-    states : foxes.core.States
+    states: foxes.core.States
         The original states
-    n_pop : int
+    n_pop: int
         The population size
 
+    :group: opt.core
+
     """
 
     def __init__(self, states, n_pop):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        states: foxes.core.States
+            The original states
+        n_pop: int
+            The population size
+
+        """
         super().__init__()
         self.states = states
         self.n_pop = n_pop
 
-    def initialize(self, algo, verbosity=0):
+    def load_data(self, algo, verbosity=0):
         """
-        Initializes the model.
+        Load and/or create all model data that is subject to chunking.
 
-        This includes loading all required data from files. The model
-        should return all array type data as part of the idata return
-        dictionary (and not store it under self, for memory reasons). This
-        data will then be chunked and provided as part of the mdata object
-        during calculations.
+        Such data should not be stored under self, for memory reasons. The
+        data returned here will automatically be chunked and then provided
+        as part of the mdata object during calculations.
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         Returns
         -------
-        idata : dict
+        idata: dict
             The dict has exactly two entries: `data_vars`,
             a dict with entries `name_str -> (dim_tuple, data_ndarray)`;
             and `coords`, a dict with entries `dim_name_str -> dim_array`
 
         """
-        self.STATE0 = self.var(FV.STATE + "0")
+        self.STATE0 = self.var(FC.STATE + "0")
         self.SMAP = self.var("SMAP")
 
-        idata = super().initialize(algo, verbosity)
-        self._update_idata(algo, idata)
-
-        if not self.states.name in algo._idata_mem:
-            raise KeyError(f"States idata '{self.states.name}' not found in algo idata memory")
-        else:
-            idata0 = algo._idata_mem[self.states.name]
-
+        idata = super().load_data(algo, verbosity)
+        idata0 = algo.get_model_data(self.states)
         for cname, coord in idata0["coords"].items():
-            if cname != FV.STATE:
+            if cname != FC.STATE:
                 idata["coords"][cname] = coord
             else:
                 idata["coords"][self.STATE0] = coord
 
         for dname, (dims0, data0) in idata0["data_vars"].items():
             if dname != FV.WEIGHT:
-                hdims = tuple([d if d != FV.STATE else self.STATE0 for d in dims0])
+                hdims = tuple([d if d != FC.STATE else self.STATE0 for d in dims0])
                 idata["data_vars"][dname] = (hdims, data0)
 
         smap = np.zeros((self.n_pop, self.states.size()), dtype=np.int32)
         smap[:] = np.arange(self.states.size())[None, :]
         smap = smap.reshape(self.size())
-        idata["data_vars"][self.SMAP] = ((FV.STATE,), smap)
+        idata["data_vars"][self.SMAP] = ((FC.STATE,), smap)
 
         found = False
         for dname, (dims0, data0) in idata["data_vars"].items():
@@ -94,6 +92,22 @@ class PopStates(States):
 
         return idata
 
+    def initialize(self, algo, verbosity=0):
+        """
+        Initializes the model.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        if not self.states.initialized:
+            self.states.initialize(algo, verbosity)
+        super().initialize(algo, verbosity)
+
     def size(self):
         """
         The total number of states.
@@ -112,12 +126,12 @@ class PopStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        weights : numpy.ndarray
+        weights: numpy.ndarray
             The weights, shape: (n_states, n_turbines)
 
         """
@@ -133,12 +147,12 @@ class PopStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
 
         Returns
         -------
-        output_vars : list of str
+        output_vars: list of str
             The output variable names
 
         """
@@ -153,18 +167,18 @@ class PopStates(States):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm
+        algo: foxes.core.Algorithm
             The calculation algorithm
-        mdata : foxes.core.Data
+        mdata: foxes.core.Data
             The model data
-        fdata : foxes.core.Data
+        fdata: foxes.core.Data
             The farm data
-        pdata : foxes.core.Data
+        pdata: foxes.core.Data
             The point data
 
         Returns
         -------
-        results : dict
+        results: dict
             The resulting data, keys: output variable str.
             Values: numpy.ndarray with shape (n_states, n_points)
 
@@ -179,7 +193,7 @@ class PopStates(States):
                 pass
             elif dms[0] == self.STATE0:
                 hdata[dname] = data[smap]
-                hdims[dname] = tuple([FV.STATE] + list(dms)[1:])
+                hdims[dname] = tuple([FC.STATE] + list(dms)[1:])
             elif self.STATE0 in dms:
                 raise ValueError(
                     f"States '{self.name}': Found states variable not at dimension 0 for mdata entry '{dname}': {dms}"
