@@ -1,5 +1,7 @@
 import numpy as np
 import pandas as pd
+from cycler import cycler
+import matplotlib.pyplot as plt
 
 from .output import Output
 import foxes.variables as FV
@@ -13,19 +15,25 @@ class FarmResultsEval(Output):
     This sums over turbines and/or states,
     given the state-turbine farm_calc results.
 
-    Parameters
-    ----------
-    farm_results : xarray.Dataset
-        The farm results
-
     Attributes
     ----------
-    results : xarray.Dataset
+    results: xarray.Dataset
         The farm results
 
+    :group: output
+
     """
 
     def __init__(self, farm_results):
+        """
+        Constructor.
+
+        Parameters
+        ----------
+        farm_results: xarray.Dataset
+            The farm results
+
+        """
         self.results = farm_results
 
     def weinsum(self, rhs, *vars):
@@ -37,17 +45,17 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        rhs : str
+        rhs: str
             The right-hand side of the einsum expression.
             Convention: 's' for states, 't' for turbines
-        vars : tuple of str or np.ndarray
+        vars: tuple of str or np.ndarray
             The variables mentioned in the expression,
             but without the obligatory weights that will
             be added at the end
 
         Returns
         -------
-        result : np.ndarray
+        result: np.ndarray
             The results array
 
         """
@@ -66,7 +74,6 @@ class FarmResultsEval(Output):
         expr = ",".join(inds) + "->" + rhs
 
         if np.any(nas):
-
             sel = ~np.any(nas, axis=1)
             fields = [f[sel] for f in fields]
 
@@ -88,18 +95,18 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars_op : dict
+        vars_op: dict
             The operation per variable. Key: str, the variable
             name. Value: str, the operation, choices
             are: sum, mean, min, max.
 
         Returns
         -------
-        data : pandas.DataFrame
+        data: pandas.DataFrame
             The results per turbine
 
         """
-        n_turbines = self.results.dims[FV.TURBINE]
+        n_turbines = self.results.sizes[FC.TURBINE]
 
         rdata = {}
         for v, op in vars_op.items():
@@ -123,7 +130,7 @@ class FarmResultsEval(Output):
                 )
 
         data = pd.DataFrame(index=range(n_turbines), data=rdata)
-        data.index.name = FV.TURBINE
+        data.index.name = FC.TURBINE
 
         return data
 
@@ -133,18 +140,18 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars_op : dict
+        vars_op: dict
             The operation per variable. Key: str, the variable
             name. Value: str, the operation, choices
             are: sum, mean, min, max.
 
         Returns
         -------
-        data : pandas.DataFrame
+        data: pandas.DataFrame
             The results per state
 
         """
-        states = self.results.coords[FV.STATE].to_numpy()
+        states = self.results.coords[FC.STATE].to_numpy()
 
         rdata = {}
         for v, op in vars_op.items():
@@ -165,7 +172,7 @@ class FarmResultsEval(Output):
                 )
 
         data = pd.DataFrame(index=states, data=rdata)
-        data.index.name = FV.STATE
+        data.index.name = FC.STATE
 
         return data
 
@@ -175,12 +182,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        states_op : dict
+        states_op: dict
             The states contraction operations.
             Key: str, the variable name. Value:
             str, the operation, choices are:
             sum, mean, min, max.
-        turbines_op : dict
+        turbines_op: dict
             The turbines contraction operations.
             Key: str, the variable name. Value:
             str, the operation, choices are:
@@ -188,7 +195,7 @@ class FarmResultsEval(Output):
 
         Returns
         -------
-        data : dict
+        data: dict
             The fully contracted results
 
         """
@@ -225,12 +232,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars : list of str
+        vars: list of str
             The variables
 
         Returns
         -------
-        data : pandas.DataFrame
+        data: pandas.DataFrame
             The results per turbine
 
         """
@@ -244,12 +251,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars : list of str
+        vars: list of str
             The variables
 
         Returns
         -------
-        data : pandas.DataFrame
+        data: pandas.DataFrame
             The results per turbine
 
         """
@@ -274,12 +281,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars : list of str
+        vars: list of str
             The variables
 
         Returns
         -------
-        data : pandas.DataFrame
+        data: pandas.DataFrame
             The results per state
 
         """
@@ -291,12 +298,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars : list of str
+        vars: list of str
             The variables
 
         Returns
         -------
-        data : pandas.DataFrame
+        data: pandas.DataFrame
             The results per state
 
         """
@@ -308,12 +315,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars : list of str
+        vars: list of str
             The variables
 
         Returns
         -------
-        data : dict
+        data: dict
             The fully contracted results
 
         """
@@ -326,12 +333,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        vars : list of str
+        vars: list of str
             The variables
 
         Returns
         -------
-        data : dict
+        data: dict
             The fully contracted results
 
         """
@@ -344,12 +351,12 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        ambient : bool
+        ambient: bool
             Flag for ambient power
 
         Returns
         -------
-        data : float
+        data: float
             The mean wind farm power
 
         """
@@ -359,28 +366,32 @@ class FarmResultsEval(Output):
 
     def calc_turbine_yield(
         self,
+        algo=None,
         annual=False,
         ambient=False,
         hours=None,
         delta_t=None,
-        P_unit_W=1e3,
+        P_unit_W=None,
     ):
         """
         Calculates the yield per turbine
 
         Parameters
         ----------
-        annual : bool, optional
+        algo: foxes.core.Algorithm, optional
+            The algorithm, for P_nominal lookup
+        annual: bool, optional
             Flag for returing annual results, by default False
-        ambient : bool, optional
+        ambient: bool, optional
             Flag for ambient power, by default False
-        hours : int, optional
+        hours: int, optional
             The duration time in hours, if not timeseries states
-        delta_t : np.datetime64, optional
+        delta_t: np.datetime64, optional
             The time delta step in case of time series data,
             by default automatically determined
-        P_unit_W : float
-            The power unit in Watts, 1000 for kW
+        P_unit_W: float
+            The power unit in Watts, 1000 for kW. Looked up
+            in algorithm if not given
 
         Returns
         -------
@@ -395,11 +406,21 @@ class FarmResultsEval(Output):
             var_in = FV.P
             var_out = FV.YLD
 
+        if algo is not None and P_unit_W is None:
+            P_unit_W = np.array(
+                [FC.P_UNITS[t.P_unit] for t in algo.farm_controller.turbine_types],
+                dtype=FC.DTYPE,
+            )[:, None]
+        elif algo is None and P_unit_W is not None:
+            pass
+        else:
+            raise KeyError("Expecting either 'algo' or 'P_unit_W'")
+
         # compute yield per turbine
-        if np.issubdtype(self.results[FV.STATE].dtype, np.datetime64):
+        if np.issubdtype(self.results[FC.STATE].dtype, np.datetime64):
             if hours is not None:
                 raise KeyError("Unexpected parameter 'hours' for timeseries data")
-            times = self.results[FV.STATE].to_numpy()
+            times = self.results[FC.STATE].to_numpy()
             if delta_t is None:
                 delta_t = times[-1] - times[-2]
             duration = times[-1] - times[0] + delta_t
@@ -428,13 +449,13 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        algo : foxes.core.Algorithm, optional
+        algo: foxes.core.Algorithm, optional
             The algorithm, for nominal power calculation
-        P_nom : list of float, optional
+        P_nom: list of float, optional
             Nominal power values for each turbine, if algo not given
-        ambient : bool, optional
+        ambient: bool, optional
             Flag for calculating ambient capacity, by default False
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         """
@@ -472,22 +493,22 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        turbine_yield : pandas.DataFrame, optional
+        turbine_yield: pandas.DataFrame, optional
             Yield values by turbine
-        power_uncert : float, optional
+        power_uncert: float, optional
             Uncertainty in the power value. Triggers
             P75 and P90 outputs
-        kwargs : dict, optional
+        kwargs: dict, optional
             Parameters for calc_turbine_yield(). Apply if
             turbine_yield is not given
 
         Returns
         -------
-        farm_yield : float
+        farm_yield: float
             Farm yield result, same unit as turbine yield
-        P75 : float, optional
+        P75: float, optional
             The P75 value, same unit as turbine yield
-        P90 : float, optional
+        P90: float, optional
             The P90 value, same unit as turbine yield
 
         """
@@ -510,7 +531,7 @@ class FarmResultsEval(Output):
 
         Parameters
         ----------
-        verbosity : int
+        verbosity: int
             The verbosity level, 0 = silent
 
         """
@@ -526,10 +547,83 @@ class FarmResultsEval(Output):
 
         Returns
         -------
-        eff : float
+        eff: float
             The farm efficiency
 
         """
         P = self.calc_mean_farm_power()
         P0 = self.calc_mean_farm_power(ambient=True) + 1e-14
         return P / P0
+
+    def gen_stdata(
+        self,
+        turbines,
+        variable,
+        fig=None,
+        ax=None,
+        figsize=None,
+        legloc="lower right",
+        animated=True,
+        ret_im=True,
+    ):
+        """
+        Generates state-turbine data,
+        intended to be used in animations
+
+        Parameters
+        ----------
+        turbines: list of int
+            The turbines for which to scatter data
+        variable: str
+            The variable name
+        fig: plt.Figure, optional
+            The figure object
+        ax: plt.Axes, optional
+            The figure axes
+        figsize: tuple, optional
+            The figsize for plt.Figure
+        legloc: str
+            The legend location
+        animated: bool
+            Flag for animated output
+        ret_im: bool
+            Flag for image return,
+
+        Yields
+        ------
+        fig: matplotlib.Figure
+            The figure object
+        im: List of matplotlib.collections.PathCollection, optional
+            The scatter artists
+
+        """
+
+        if fig is None:
+            hfig = plt.figure(figsize=figsize)
+        else:
+            hfig = fig
+        if ax is None:
+            hax = hfig.add_subplot(111)
+        else:
+            hax = ax
+
+        hax.set_xlabel(f"State")
+        hax.set_ylabel(variable)
+        cc = cycler(color="bgrcmyk")
+
+        data = self.results[variable].to_numpy()
+        hasl = set()
+        for si in range(len(data)):
+            im = []
+            hax.set_prop_cycle(cc)
+            for ti in turbines:
+                lbl = None if ti in hasl else f"Turbine {ti}"
+                im += hax.plot(range(si), data[:si, ti], label=lbl, animated=animated)
+                hasl.add(ti)
+
+            hax.legend(loc=legloc)
+
+            if ret_im:
+                yield hfig, im
+            else:
+                yield hfig
