@@ -1,9 +1,9 @@
 import numpy as np
 import pandas as pd
-import plotly.express as px
 
 import foxes.variables as FV
-from foxes.utils import wd2uv, uv2wd
+import foxes.constants as FC
+from foxes.utils import wd2uv, uv2wd, TabWindroseAxes
 from foxes.algorithms import Downwind
 from foxes.core import WindFarm, Turbine
 from foxes.models import ModelBook
@@ -12,30 +12,35 @@ from .output import Output
 
 class RosePlotOutput(Output):
     """
-    Class for rose plot creation
+     Class for rose plot creation
 
-    Parameters
-    ----------
-    results : xarray.Dataset
-        The calculation results (farm or points)
+     Attributes
+     ----------
+     results: pandas.DataFrame
+         The calculation results (farm or points)
 
-    Attributes
-    ----------
-    results : pandas.DataFrame
-        The calculation results (farm or points)
+    :group: output
 
     """
 
     def __init__(self, results):
+        """
+        Constructor.
 
-        dims = list(results.dims.keys())
-        if dims[1] == FV.TURBINE:
-            self._rtype = FV.TURBINE
-        elif dims[1] == FV.POINT:
-            self._rtype = FV.POINT
+        Parameters
+        ----------
+        results: xarray.Dataset
+            The calculation results (farm or points)
+
+        """
+        dims = list(results.sizes.keys())
+        if dims[1] == FC.TURBINE:
+            self._rtype = FC.TURBINE
+        elif dims[1] == FC.POINT:
+            self._rtype = FC.POINT
         else:
             raise KeyError(
-                f"Results dimension 1 is neither '{FV.TURBINE}' nor '{FV.POINT}': dims = {results.dims}"
+                f"Results dimension 1 is neither '{FC.TURBINE}' nor '{FC.POINT}': dims = {results.dims}"
             )
 
         self.results = results.to_dataframe()
@@ -47,14 +52,14 @@ class RosePlotOutput(Output):
 
         Parameters
         ----------
-        dname : str
+        dname: str
             The variable name
 
         Returns
         -------
-        title : str
+        title: str
             The long name of the variable
-        legend : str
+        legend: str
             The legend/axis text
 
         """
@@ -101,43 +106,44 @@ class RosePlotOutput(Output):
         wd_var=FV.AMB_WD,
         turbine=None,
         point=None,
-        legend=None,
+        start0=False,
     ):
         """
         Get pandas DataFrame with wind rose data.
 
         Parameters
         ----------
-        sectors : int
+        sectors: int
             The number of wind direction sectors
-        var : str
+        var: str
             The data variable name
-        var_bins : list of float
+        var_bins: list of float
             The variable bin seperation values
-        wd_var : str, optional
+        wd_var: str, optional
             The wind direction variable name
-        turbine : int, optional
+        turbine: int, optional
             Only relevant in case of farm results.
             If None, mean over all turbines.
             Else, data from a single turbine
-        point : int, optional
+        point: int, optional
             Only relevant in case of point results.
             If None, mean over all points.
             Else, data from a single point
-        legend : str, optional
-            The data legend string
+        start0: bool
+            Flag for starting the first sector at
+            zero degrees instead of minus half width
 
         Returns
         -------
-        pd.DataFrame :
+        pd.DataFrame:
             The wind rose data
 
         """
 
         dwd = 360.0 / sectors
         wds = np.arange(0.0, 360.0, dwd)
-        wdb = np.arange(-dwd / 2, 360.0, dwd)
-        lgd = legend if legend is not None else var
+        wdb = np.append(wds, 360) if start0 else np.arange(-dwd / 2, 360.0, dwd)
+        lgd = f"interval_{var}"
 
         data = self.results[[wd_var, FV.WEIGHT]].copy()
         data[lgd] = self.results[var]
@@ -148,7 +154,7 @@ class RosePlotOutput(Output):
         data[FV.WEIGHT] *= 100
         data = data.rename(columns={FV.WEIGHT: "frequency"})
 
-        el = turbine if self._rtype == FV.TURBINE else point
+        el = turbine if self._rtype == FC.TURBINE else point
         if el is None:
             data = data.groupby(level=0).mean()
         else:
@@ -158,14 +164,43 @@ class RosePlotOutput(Output):
 
         data["wd"] = uv2wd(data[["u", "v"]].to_numpy())
         data.drop(["u", "v"], axis=1, inplace=True)
-        data.loc[data["wd"] > 360.0 - dwd / 2, "wd"] -= 360.0
+        if not start0:
+            data.loc[data["wd"] > 360.0 - dwd / 2, "wd"] -= 360.0
 
         data[wd_var] = pd.cut(data["wd"], wdb, labels=wds)
         data[lgd] = pd.cut(data[lgd], var_bins, right=False, include_lowest=True)
 
-        grp = data[[wd_var, lgd, "frequency"]].groupby([wd_var, lgd])
+        grp = data[[wd_var, lgd, "frequency"]].groupby([wd_var, lgd], observed=False)
         data = grp.sum().reset_index()
 
+        data[wd_var] = data[wd_var].astype(np.float64)
+        data[lgd] = list(data[lgd])
+        if start0:
+            data[wd_var] += dwd / 2
+
+        ii = pd.IntervalIndex(data[lgd])
+        data[var] = ii.mid
+        data[f"bin_min_{var}"] = ii.left
+        data[f"bin_max_{var}"] = ii.right
+        data[f"bin_min_{wd_var}"] = data[wd_var] - dwd / 2
+        data[f"bin_max_{wd_var}"] = data[wd_var] + dwd / 2
+        data["sector"] = (data[wd_var] / dwd).astype(int)
+
+        data = data[
+            [
+                wd_var,
+                var,
+                "sector",
+                f"bin_min_{wd_var}",
+                f"bin_max_{wd_var}",
+                f"bin_min_{var}",
+                f"bin_max_{var}",
+                lgd,
+                "frequency",
+            ]
+        ]
+        data.index.name = "bin"
+
         return data
 
     def get_figure(
@@ -176,53 +211,72 @@ class RosePlotOutput(Output):
         wd_var=FV.AMB_WD,
         turbine=None,
         point=None,
-        cmap="Turbo",
         title=None,
         legend=None,
-        layout_dict={},
-        title_dict={},
+        design="bar",
+        start0=False,
+        fig=None,
+        figsize=None,
+        rect=None,
+        ret_data=False,
+        **kwargs,
     ):
         """
-        Creates px figure object
+        Creates figure object
 
         Parameters
         ----------
-        sectors : int
+        sectors: int
             The number of wind direction sectors
-        var : str
+        var: str
             The data variable name
-        var_bins : list of float
+        var_bins: list of float
             The variable bin seperation values
-        wd_var : str, optional
+        wd_var: str, optional
             The wind direction variable name
-        turbine : int, optional
+        turbine: int, optional
             Only relevant in case of farm results.
             If None, mean over all turbines.
             Else, data from a single turbine
-        point : int, optional
+        point: int, optional
             Only relevant in case of point results.
             If None, mean over all points.
             Else, data from a single point
-        legend : str, optional
+        title. str, optional
+            The title
+        legend: str, optional
             The data legend string
-        layout_dict : dict, optional
-            Optional parameters for the px figure layout
-        title_dict : dict, optional
-            Optional parameters for the px title layout
+        design: str
+            The wind rose design: bar, contour, ...
+        start0: bool
+            Flag for starting the first sector at
+            zero degrees instead of minus half width
+        fig: matplotlib.Figure
+            The figure to which to add an axis
+        figsize: tuple, optional
+            The figsize of the newly created figure
+        rect: list, optional
+            The rectangle of the figure which to fill,
+            e.g. [0.1, 0.1, 0.8, 0.8]
+        ret_data: bool
+            Flag for returning wind rose data
+        kwargs: dict, optional
+            Additional arguments for TabWindroseAxes
+            plot function
 
         Returns
         -------
-        px.Figure :
+        fig: matplotlib.pyplot.Figure
             The rose plot figure
+        data: pd.DataFrame, optional
+            The wind rose data
 
         """
-
+        lg = legend
         if title is None or legend is None:
             ttl, lg = self.get_data_info(var)
         if title is not None:
             ttl = title
-        if legend is not None:
-            lg = legend
 
         wrdata = self.get_data(
             sectors=sectors,
@@ -231,26 +285,29 @@ class RosePlotOutput(Output):
             wd_var=wd_var,
             turbine=turbine,
             point=point,
-            legend=lg,
+            start0=start0,
         )
 
-        cols = px.colors.sequential.__dict__.keys()
-        cols = [c for c in cols if isinstance(px.colors.sequential.__dict__[c], list)]
-        cmap = px.colors.sequential.__dict__[cols[cols.index(cmap)]]
+        ax = TabWindroseAxes.from_ax(fig=fig, rect=rect, figsize=figsize)
+        fig = ax.get_figure()
 
-        fig = px.bar_polar(
-            wrdata, r="frequency", theta=wd_var, color=lg, color_discrete_sequence=cmap
+        plfun = getattr(ax, design)
+        plfun(
+            direction=wrdata[wd_var].to_numpy(),
+            var=wrdata[var].to_numpy(),
+            weights=wrdata["frequency"].to_numpy(),
+            bin_min_dir=np.sort(wrdata[f"bin_min_{wd_var}"].unique()),
+            bin_min_var=np.sort(wrdata[f"bin_min_{var}"].unique()),
+            bin_max_var=np.sort(wrdata[f"bin_max_{var}"].unique()),
+            **kwargs,
         )
+        ax.set_legend(title=lg)
+        ax.set_title(ttl)
 
-        tdict = dict(xanchor="center", yanchor="top", x=0.5, y=0.97)
-        tdict.update(title_dict)
-
-        ldict = dict(width=500, height=400, margin=dict(l=50, r=50, t=50, b=50))
-        ldict.update(layout_dict)
-
-        fig = fig.update_layout(title={"text": ttl, **tdict}, **ldict)
-
-        return fig
+        if ret_data:
+            return fig, wrdata
+        else:
+            return fig
 
     def write_figure(
         self,
@@ -258,81 +315,66 @@ class RosePlotOutput(Output):
         sectors,
         var,
         var_bins,
-        wd_var=FV.AMB_WD,
-        turbine=None,
-        point=None,
-        cmap="Turbo",
-        title=None,
-        legend=None,
-        layout_dict={},
-        title_dict={},
+        ret_data=False,
+        **kwargs,
     ):
         """
         Write rose plot to file
 
         Parameters
         ----------
-        file_name : str
+        file_name: str
             Path to the output file
-        sectors : int
+        sectors: int
             The number of wind direction sectors
-        var : str
+        var: str
             The data variable name
-        var_bins : list of float
+        var_bins: list of float
             The variable bin seperation values
-        wd_var : str, optional
-            The wind direction variable name
-        turbine : int, optional
-            Only relevant in case of farm results.
-            If None, mean over all turbines.
-            Else, data from a single turbine
-        point : int, optional
-            Only relevant in case of point results.
-            If None, mean over all points.
-            Else, data from a single point
-        legend : str, optional
-            The data legend string
-        layout_dict : dict, optional
-            Optional parameters for the px figure layout
-        title_dict : dict, optional
-            Optional parameters for the px title layout
+        ret_data: bool
+            Flag for returning wind rose data
+        kwargs: dict, optional
+            Additional parameters for get_figure()
+
+        Returns
+        -------
+        data: pd.DataFrame, optional
+            The wind rose data
 
         """
 
-        fig = self.get_figure(
+        r = self.get_figure(
             sectors=sectors,
             var=var,
             var_bins=var_bins,
-            wd_var=wd_var,
-            turbine=turbine,
-            point=point,
-            cmap=cmap,
-            title=title,
-            legend=legend,
-            layout_dict=layout_dict,
-            title_dict=title_dict,
+            ret_data=ret_data,
+            **kwargs,
         )
-
-        fig.write_image(file_name)
+        if ret_data:
+            r[0].write_image(file_name)
+            return r[1]
+        else:
+            r.write_image(file_name)
 
 
 class StatesRosePlotOutput(RosePlotOutput):
     """
-    Class for rose plot creation directly from states
+     Class for rose plot creation directly from states
 
-    Parameters
-    ----------
-    states : foxes.core.States
-        The states from which to compute the wind rose
-    point : numpy.ndarray
-        The evaluation point, shape: (3,)
-    mbook : foxes.models.ModelBook, optional
-        The model book
+     Parameters
+     ----------
+     states: foxes.core.States
+         The states from which to compute the wind rose
+     point: numpy.ndarray
+         The evaluation point, shape: (3,)
+     mbook: foxes.models.ModelBook, optional
+         The model book
+
+    :group: output
 
     """
 
     def __init__(self, states, point, mbook=None, ws_var=FV.AMB_REWS):
-
         farm = WindFarm()
         farm.add_turbine(
             Turbine(
@@ -344,7 +386,7 @@ class StatesRosePlotOutput(RosePlotOutput):
         )
 
         mbook = mbook if mbook is not None else ModelBook()
-        algo = Downwind(mbook, farm, states, wake_models=[], verbosity=0)
+        algo = Downwind(farm, states, wake_models=[], mbook=mbook, verbosity=0)
 
         results = algo.calc_farm(ambient=True).rename_vars({ws_var: FV.AMB_WS})
 
